@@ -24,9 +24,17 @@ Application module.
 Handles application initialization, shutdown, opening projects, autosave and changing
 sequences.
 """
+try:
+    import pgi
+    pgi.install_as_gi()
+except ImportError:
+    pass
+    
+import gi
 
 from gi.repository import GObject
 from gi.repository import GLib
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 from gi.repository import Gdk
 
@@ -75,7 +83,11 @@ import renderconsumer
 import respaths
 import resync
 import sequence
+import snapping
+import titler
 import tlinewidgets
+import toolsintegration
+import toolnatron
 import trimmodes
 import translations
 import undo
@@ -99,14 +111,20 @@ exit_timeout_id = -1
 window_resize_id = -1
 window_state_id = -1
 
-logger = None
+_log_file = None
 
+assoc_file_path = None
+assoc_timeout_id = None
 
 def main(root_path):
     """
     Called at application start.
     Initializes application with a default project.
     """
+    # DEBUG: Direct output to log file if log file set
+    if _log_file != None:
+        log_print_output_to_file()
+    
     # Print OS, Python version and GTK+ version
     try:
         os_release_file = open("/etc/os-release","r")
@@ -129,6 +147,7 @@ def main(root_path):
 
     # passing -xdg as a flag will change the user_dir location with XDG_CONFIG_HOME
     # For full xdg-app support all the launch processes need to add this too, currently not impl.
+
     for arg in sys.argv:
         if arg.lower() == "-xdg":
             editorstate.use_xdg = True
@@ -150,7 +169,13 @@ def main(root_path):
         os.mkdir(utils.get_hidden_screenshot_dir_path())
     if not os.path.exists(user_dir + appconsts.GMIC_DIR):
         os.mkdir(user_dir + appconsts.GMIC_DIR)
-
+    if not os.path.exists(user_dir + appconsts.MATCH_FRAME_DIR):
+        os.mkdir(user_dir + appconsts.MATCH_FRAME_DIR)
+    if not os.path.exists(user_dir + appconsts.TRIM_VIEW_DIR):
+        os.mkdir(user_dir + appconsts.TRIM_VIEW_DIR)
+    if not os.path.exists(user_dir + appconsts.NATRON_DIR):
+        os.mkdir(user_dir + appconsts.NATRON_DIR)
+        
     # Set paths.
     respaths.set_paths(root_path)
 
@@ -162,6 +187,7 @@ def main(root_path):
         editorstate.display_all_audio_levels = False
     editorpersistance.create_thumbs_folder_if_needed(user_dir)
     editorpersistance.create_rendered_clips_folder_if_needed(user_dir)
+
     editorpersistance.save()
 
     # Init translations module with translations data
@@ -177,7 +203,7 @@ def main(root_path):
     Gdk.threads_init()
     Gdk.threads_enter()
 
-    # Request dark them if so desired
+    # Request dark theme if so desired
     if editorpersistance.prefs.dark_theme == True:
         Gtk.Settings.get_default().set_property("gtk-application-prefer-dark-theme", True)
 
@@ -190,6 +216,7 @@ def main(root_path):
     editorstate.SCREEN_WIDTH = scr_w
     editorstate.SCREEN_HEIGHT = scr_h
 
+    print scr_w, scr_h
     print "Small height:", editorstate.screen_size_small_height()
     print "Small width:",  editorstate.screen_size_small_width()
 
@@ -224,31 +251,26 @@ def main(root_path):
     # Create list of available mlt profiles
     mltprofiles.load_profile_list()
     
-    # Launch association file if found in arguments
-    launch_file_path = get_assoc_file_path()
-    if launch_file_path != None:
-        try:
-            print "Launching assoc file:" +  launch_file_path
-            persistance.show_messages = False
-            editorstate.project = persistance.load_project(launch_file_path)
-            persistance.show_messages = True
-            check_crash = False
-        except:
-            editorstate.project = projectdata.get_default_project()
-            persistance.show_messages = True
-            check_crash = True
-    else:
-        # There is always a project open, so at startup we create a default project.
-        # Set default project as the project being edited.
-        editorstate.project = projectdata.get_default_project()
-        check_crash = True
+    # Save assoc file path if found in arguments
+    global assoc_file_path
+    assoc_file_path = get_assoc_file_path()
+        
+    # There is always a project open, so at startup we create a default project.
+    # Set default project as the project being edited.
+    editorstate.project = projectdata.get_default_project()
+    check_crash = True
 
     # Audiomonitoring being available needs to be known before GUI creation
     audiomonitoring.init(editorstate.project.profile)
 
-    # Check for gmic
-    gmic.test_availablity()
+    # Set trim view mode to current default value
+    editorstate.show_trim_view = editorpersistance.prefs.trim_view_default
 
+    # Check for tools and init tools integration
+    gmic.test_availablity()
+    toolnatron.init()
+    toolsintegration.init()
+    
     # Create player object
     create_player()
 
@@ -276,7 +298,7 @@ def main(root_path):
     # Get existing autosave files
     autosave_files = get_autosave_files()
 
-    # Show splash
+    # Clear splash
     if ((editorpersistance.prefs.display_splash_screen == True) and len(autosave_files) == 0):
         global splash_timeout_id
         splash_timeout_id = GLib.timeout_add(2600, destroy_splash_screen)
@@ -299,7 +321,13 @@ def main(root_path):
     # We prefer to monkeypatch some callbacks into some modules, usually to
     # maintain a simpler and/or non-circular import structure
     monkeypatch_callbacks()
-    
+
+    if not(check_crash == True and len(autosave_files) > 0):
+        if assoc_file_path != None:
+            print "Launch assoc file:", assoc_file_path
+            global assoc_timeout_id
+            assoc_timeout_id = GObject.timeout_add(10, open_assoc_file)
+            
     # Launch gtk+ main loop
     Gtk.main()
 
@@ -334,7 +362,11 @@ def monkeypatch_callbacks():
     
     # Posionbar in gmic.py doesnot need trimmodes.py dependency and is avoided 
     positionbar.trimmodes_set_no_edit_trim_mode = trimmodes.set_no_edit_trim_mode
-    
+
+    # Snapping is done in a separate module but needs some tlinewidgets state info
+    snapping._get_frame_for_x_func = tlinewidgets.get_frame
+    snapping._get_x_for_frame_func = tlinewidgets._get_frame_x
+
     # These provide clues for further module refactoring 
 
 # ---------------------------------- program, sequence and project init
@@ -344,17 +376,19 @@ def get_assoc_file_path():
     """
     arg_str = ""
     for arg in sys.argv:
-        arg_str = arg
+        ext_index = arg.find(".flb")
+        if ext_index != -1:
+            arg_str = arg
     
     if len(arg_str) == 0:
         return None
-    
-    ext_index = arg_str.find(".flb")
-    if ext_index == -1:
-        return None
     else:
         return arg_str
 
+def open_assoc_file():
+    GObject.source_remove(assoc_timeout_id)
+    projectaction.actually_load_project(assoc_file_path, block_recent_files=False)
+    
 def create_gui():
     """
     Called at app start to create gui objects and handles for them.
@@ -364,6 +398,12 @@ def create_gui():
     updater.set_clip_edit_mode_callback = editevent.set_clip_monitor_edit_mode
     updater.load_icons()
 
+    # Notebook indexes are differn for 1 and 2 window layouts
+    if editorpersistance.prefs.global_layout != appconsts.SINGLE_WINDOW:
+        medialog.range_log_notebook_index = 0
+        compositeeditor.compositor_notebook_index = 2
+        clipeffectseditor.filters_notebook_index = 1
+
     # Create window and all child components
     editor_window = editorwindow.EditorWindow()
     
@@ -377,7 +417,9 @@ def create_gui():
     
     # Connect window global key listener
     gui.editor_window.window.connect("key-press-event", keyevents.key_down)
-    
+    if editorpersistance.prefs.global_layout != appconsts.SINGLE_WINDOW:
+        gui.editor_window.window2.connect("key-press-event", keyevents.key_down)
+
     # Give undo a reference to uimanager for menuitem state changes
     undo.set_menu_items(gui.editor_window.uimanager)
     
@@ -437,6 +479,8 @@ def init_project_gui():
     gui.tline_left_corner.update_gui()
     projectinfogui.update_project_info()
 
+    titler.reset_titler()
+
     # Set render folder selector to last render if prefs require 
     folder_path = editorstate.PROJECT().get_last_render_folder()
     if folder_path != None and editorpersistance.prefs.remember_last_render_dir == True:
@@ -539,6 +583,9 @@ def open_project(new_project):
         loaded_autosave_file = None
 
     editorstate.update_current_proxy_paths()
+    editorstate.fade_length = -1
+    editorstate.transition_length = -1
+    editorstate.clear_trim_clip_cache()
     audiomonitoring.init_for_project_load()
     updater.window_resized()
 
@@ -546,6 +593,11 @@ def open_project(new_project):
     gui.editor_window.window.handler_unblock(window_state_id)
     start_autosave()
 
+    if new_project.update_media_lengths_on_load == True:
+        projectaction.update_media_lengths()
+
+    #editorstate.project.c_seq.print_all()
+        
 def change_current_sequence(index):
     stop_autosave()
     editorstate.project.c_seq = editorstate.project.sequences[index]
@@ -689,6 +741,11 @@ def _set_draw_params():
         
     if editorstate.screen_size_small_height() == True:
         appconsts.TOP_ROW_HEIGHT = 10
+        projectinfogui.PROJECT_INFO_PANEL_HEIGHT = 140
+
+    if editorstate.SCREEN_WIDTH < 1153 and editorstate.SCREEN_HEIGHT < 865:
+        editorwindow.MONITOR_AREA_WIDTH = 400
+        positionbar.BAR_WIDTH = 100
 
 def _too_small_screen_exit():
     global exit_timeout_id
@@ -710,24 +767,14 @@ def _early_exit(dialog, response):
     # Exit gtk main loop.
     Gtk.main_quit() 
 
-
 # ------------------------------------------------------- logging
-def init_logger():
-    try:
-        import logging
-        global logger
-        logger = logging.getLogger('flowblade')
-        hdlr = logging.FileHandler('/home/janne/flog')
-        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
-        hdlr.setFormatter(formatter)
-        logger.addHandler(hdlr)
-        logger.setLevel(logging.INFO)
-    except:
-        print "logging failed"
+def log_print_output_to_file():
+    so = se = open(_log_file, 'w', 0)
 
-def log_msg(msg):
-    global logger
-    logger.info(msg)
+    sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)
+
+    os.dup2(so.fileno(), sys.stdout.fileno())
+    os.dup2(se.fileno(), sys.stderr.fileno())
 
 # ------------------------------------------------------ shutdown
 def shutdown():
@@ -735,18 +782,7 @@ def shutdown():
     return True # Signal that event is handled, otherwise it'll destroy window anyway
 
 def get_save_time_msg():
-    if projectaction.save_time == None:
-        return _("Project has not been saved since it was opened.")
-    
-    save_ago = (time.clock() - projectaction.save_time) / 60.0
-
-    if save_ago < 1:
-        return _("Project was saved less than a minute ago.")
-
-    if save_ago < 2:
-        return _("Project was saved one minute ago.")
-    
-    return _("Project was saved ") + str(int(save_ago)) + _(" minutes ago.")
+    return projectaction.get_save_time_msg()
 
 def _shutdown_dialog_callback(dialog, response_id):
     dialog.destroy()
@@ -773,6 +809,10 @@ def _shutdown_dialog_callback(dialog, response_id):
     alloc = gui.editor_window.window.get_allocation()
     x, y, w, h = alloc.x, alloc.y, alloc.width, alloc.height 
     editorpersistance.prefs.exit_allocation = (w, h)
+    if gui.editor_window.window2 != None:
+        alloc = gui.editor_window.window2.get_allocation()
+        pos_x, pos_y = gui.editor_window.window2.get_position()
+        editorpersistance.prefs.exit_allocation_window_2 = (alloc.width, alloc.height, pos_x, pos_y)       
     editorpersistance.prefs.app_v_paned_position = gui.editor_window.app_v_paned.get_position()
     editorpersistance.prefs.top_paned_position = gui.editor_window.top_paned.get_position()
     editorpersistance.prefs.mm_paned_position = gui.editor_window.mm_paned.get_position()
@@ -781,6 +821,9 @@ def _shutdown_dialog_callback(dialog, response_id):
     # Block reconnecting consumer before setting window not visible
     updater.player_refresh_enabled = False
     gui.editor_window.window.set_visible(False)
+    if gui.editor_window.window2 != None:
+        gui.editor_window.window2.set_visible(False)
+
     # Close and destroy app when gtk finds time to do it after hiding window
     GLib.idle_add(_app_destroy)
 
