@@ -72,7 +72,19 @@ def load():
         write_file = file(recents_file_path, "wb")
         pickle.dump(recent_projects, write_file)
 
-    # version of program may have different prefs objects and 
+    # Remove non-existing projects from recents list
+    remove_list = []
+    for proj_path in recent_projects.projects:
+        if os.path.isfile(proj_path) == False:
+            remove_list.append(proj_path)
+
+    if len(remove_list) > 0:
+        for proj_path in remove_list:
+            recent_projects.projects.remove(proj_path)
+        write_file = file(recents_file_path, "wb")
+        pickle.dump(recent_projects, write_file)
+        
+    # Versions of program may have different prefs objects and 
     # we may need to to update prefs on disk if user has e.g.
     # installed later version of Flowblade
     current_prefs = EditorPreferences()
@@ -162,10 +174,11 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
 
     default_profile_combo, open_in_last_opened_check, open_in_last_rendered_check, undo_max_spin, load_order_combo = gen_opts_widgets
     
+    # Jul-2016 - SvdB - Added play_pause_button
     auto_play_in_clip_monitor_check, auto_center_check, grfx_insert_length_spin, \
-    trim_exit_click, trim_quick_enter, remember_clip_frame, overwrite_clip_drop, cover_delete = edit_prefs_widgets
+    trim_exit_click, trim_quick_enter, remember_clip_frame, overwrite_clip_drop, cover_delete, play_pause_button = edit_prefs_widgets
     
-    use_english, disp_splash, buttons_style, dark_theme, theme_combo, audio_levels_combo = view_prefs_widgets
+    use_english, disp_splash, buttons_style, dark_theme, theme_combo, audio_levels_combo, window_mode_combo = view_prefs_widgets
 
     global prefs
     prefs.open_in_last_opended_media_dir = open_in_last_opened_check.get_active()
@@ -182,6 +195,8 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     prefs.remember_monitor_clip_frame = remember_clip_frame.get_active()
     prefs.overwrite_clip_drop = (overwrite_clip_drop.get_active() == 0)
     prefs.trans_cover_delete = cover_delete.get_active()
+    # Jul-2016 - SvdB - For play/pause button
+    prefs.play_pause = play_pause_button.get_active()
     
     prefs.use_english_always = use_english.get_active()
     prefs.display_splash_screen = disp_splash.get_active()
@@ -189,6 +204,7 @@ def update_prefs_from_widgets(widgets_tuples_tuple):
     prefs.dark_theme = (dark_theme.get_active() == 1)
     prefs.theme_fallback_colors = theme_combo.get_active() 
     prefs.display_all_audio_levels = (audio_levels_combo.get_active() == 0)
+    prefs.global_layout = window_mode_combo.get_active() + 1 # +1 'cause values are 1 and 2
 
 def get_graphics_default_in_out_length():
     in_fr = int(15000/2) - int(prefs.default_grfx_length/2)
@@ -253,7 +269,16 @@ class EditorPreferences:
         self.jack_output_type = appconsts.JACK_OUT_AUDIO # not used
         self.media_load_order = appconsts.LOAD_ABSOLUTE_FIRST
         self.use_english_always = False
-        self.theme_fallback_colors = 0 # index oc gui._THEME_COLORS
+        self.theme_fallback_colors = 0 # index of gui._THEME_COLORS
         self.display_all_audio_levels = True
         self.overwrite_clip_drop = True
         self.trans_cover_delete = True
+        # Jul-2016 - SvdB - For play/pause button
+        self.play_pause = False
+        self.midbar_layout = appconsts.MIDBAR_TC_LEFT
+        self.global_layout = appconsts.SINGLE_WINDOW
+        self.trim_view_default = appconsts.TRIM_VIEW_OFF
+        self.trim_view_message_shown = False
+        self.exit_allocation_window_2 = (0, 0, 0, 0)
+        
+        
