@@ -19,13 +19,14 @@
 """
 
 """
-Module holds current editor state.
+Module holds current global editor state.
 
 Accessor methods are there mainly to improve code readability elsewhere.
 
 We're using BIG_METHOD_NAMES() for state objects. This is a bit unusual
 but looks good when reading code.
 """
+
 import appconsts
 
 # Edit modes
@@ -42,6 +43,7 @@ SLIDE_TRIM_NO_EDIT = 9
 MULTI_MOVE = 10
 CLIP_END_DRAG = 11
 
+
 # Project being edited
 project = None
 
@@ -99,10 +101,21 @@ project_is_loading = False
 
 # Audio levels display mode, False means that audio levels are displayed on request
 display_all_audio_levels = True
+display_clip_media_thumbnails = True
 
 # Flag for window being in fullscreen mode
 fullscreen = False
 
+# Trim view mode
+show_trim_view = appconsts.TRIM_VIEW_OFF
+
+# Remember fade and transition lengths
+fade_length = -1
+transition_length = -1
+
+# Trim clips cache for quicker inits, path -> clip
+_trim_clips_cache = {}
+
 def current_is_move_mode():
     if ((edit_mode == INSERT_MOVE) or (edit_mode == OVERWRITE_MOVE) or (edit_mode == MULTI_MOVE)):
         return True
@@ -151,21 +164,16 @@ def timeline_visible():
     return _timeline_displayed
 
 def mlt_version_is_equal_or_greater(test_version):
-    runtime_ver = _get_mlt_ver_num(mlt_version)
-    t_ver = _get_mlt_ver_num(test_version)
-
-    if runtime_ver >= t_ver:
-        return True
+    runtime_ver = mlt_version.split(".")
+    test_ver = test_version.split(".")
+    
+    if runtime_ver[0] >= test_ver[0]:
+        if  runtime_ver[1] >= test_ver[1]:
+            if  runtime_ver[2] >= test_ver[2]:
+                return True
     
     return False
 
-def _get_mlt_ver_num(ver):
-    parts = ver.split(".")
-    p_str = parts[0] + parts[1] + parts[2]
-    p_str =  p_str.lstrip("0")
-
-    return int(p_str)
-
 def set_copy_paste_objects(objs):
     global _copy_paste_objects
     _copy_paste_objects = objs
@@ -174,14 +182,35 @@ def get_copy_paste_objects():
     return _copy_paste_objects
 
 def screen_size_small_height():
-    if SCREEN_HEIGHT < 865:
+    if SCREEN_HEIGHT < 901:
+        return True
+    else:
+        if SCREEN_WIDTH < 1280:
+            return True
+            
+        return False
+
+def screen_size_small_width():
+    if SCREEN_WIDTH < 1368:
         return True
     else:
         return False
 
-def screen_size_small_width():
-    if SCREEN_WIDTH < 1200:
+def screen_size_small():
+    if screen_size_small_height() == True or screen_size_small_width() == True:
         return True
-    else:
-        return False
-        
+    
+    return False
+
+def get_cached_trim_clip(path):
+    try:
+        return _trim_clips_cache[path]
+    except:
+        return None 
+
+def add_cached_trim_clip(clip):
+     _trim_clips_cache[clip.path] = clip
+
+def clear_trim_clip_cache():
+    global _trim_clips_cache
+    _trim_clips_cache = {}
