@@ -51,6 +51,7 @@ import medialog
 import menuactions
 import middlebar
 import monitorevent
+import monitorwidget
 import respaths
 import render
 import rendergui
@@ -128,6 +129,13 @@ class EditorWindow:
         self.window.set_icon_from_file(respaths.IMAGE_PATH + "flowbladeappicon.png")
         self.window.set_border_width(5)
 
+        self.window2 = None
+        if editorpersistance.prefs.global_layout != appconsts.SINGLE_WINDOW:
+            self.window2 = Gtk.Window(Gtk.WindowType.TOPLEVEL)
+            self.window2.set_icon_from_file(respaths.IMAGE_PATH + "flowbladeappicon.png")
+            self.window2.set_border_width(5)
+            self.window2.connect("delete-event", lambda w, e:app.shutdown())
+
         # To ask confirmation for shutdown 
         self.window.connect("delete-event", lambda w, e:app.shutdown())
 
@@ -146,7 +154,7 @@ class EditorWindow:
             ('SaveSnapshot', None, _('Save Backup Snapshot...'), None, None, lambda a:projectaction.save_backup_snapshot()),
             ('ExportMenu', None, _('Export')),
             ('ExportMeltXML', None, _('MLT XML'), None, None, lambda a:exporting.MELT_XML_export()),
-            ('ExportEDL', None, _('EDL CMX 3600'), None, None, lambda a:exporting.EDL_export()),
+            ('ExportEDL', None, _('EDL'), None, None, lambda a:exporting.EDL_export()),
             ('ExportScreenshot', None, _('Current Frame'), None, None, lambda a:exporting.screenshot_export()),
             ('Close', None, _('_Close'), None, None, lambda a:projectaction.close_project()),
             ('Quit', None, _('_Quit'), '<control>Q', None, lambda a:app.shutdown()),
@@ -169,6 +177,9 @@ class EditorWindow:
             ('AddTransition', None, _('Add Single Track Transition'), None, None, lambda a:tlineaction.add_transition_menu_item_selected()),
             ('AddFade', None, _('Add Single Track Fade'), None, None, lambda a:tlineaction.add_fade_menu_item_selected()),
             ('ClearFilters', None, _('Clear Filters'), None, None, lambda a:clipmenuaction.clear_filters()),
+            ('Timeline', None, _('Timeline')),
+            ('FiltersOff', None, _('All Filters Off'), None, None, lambda a:tlineaction.all_filters_off()),
+            ('FiltersOn', None, _('All Filters On'), None, None, lambda a:tlineaction.all_filters_on()),
             ('SyncCompositors', None, _('Sync All Compositors'), None, None, lambda a:tlineaction.sync_all_compositors()),
             ('ChangeSequenceTracks', None, _('Change Sequence Tracks Count...'), None, None, lambda a:projectaction.change_sequence_track_count()),
             ('Watermark', None, _('Watermark...'), None, None, lambda a:menuactions.edit_watermark()),
@@ -226,6 +237,7 @@ class EditorWindow:
                     <separator/>
                     <menu action='ExportMenu'>
                         <menuitem action='ExportMeltXML'/>
+                        <menuitem action='ExportEDL'/>
                         <menuitem action='ExportScreenshot'/>
                     </menu>
                     <separator/>
@@ -255,6 +267,11 @@ class EditorWindow:
                     <menuitem action='SyncCompositors'/>
                     <menuitem action='ClearFilters'/>
                     <separator/>
+                    <menu action='Timeline'>
+                        <menuitem action='FiltersOff'/>
+                        <menuitem action='FiltersOn'/>
+                    </menu>
+                    <separator/>
                     <menuitem action='AddTransition'/>
                     <menuitem action='AddFade'/>
                     <separator/>
@@ -436,7 +453,10 @@ class EditorWindow:
             render_panel_right = rendergui.get_render_panel_right(render.widgets,
                                                                   lambda w,e: projectaction.do_rendering(),
                                                                   lambda w,e: projectaction.add_to_render_queue())
-            render_hbox = Gtk.HBox(True, 5)
+            if editorstate.screen_size_small_width() == False:
+                render_hbox = Gtk.HBox(True, 5)
+            else:
+                render_hbox = Gtk.HBox(False, 5)             
             render_hbox.pack_start(render_panel_left, True, True, 0)
             render_hbox.pack_start(render_panel_right, True, True, 0)
 
@@ -468,7 +488,6 @@ class EditorWindow:
         project_vbox = Gtk.VBox()
         project_vbox.pack_start(project_info_panel, False, True, 0)
         project_vbox.pack_start(seq_panel, True, True, 0)
-        
         project_panel = guiutils.set_margins(project_vbox, 0, 2, 6, 2)
         
         # Notebook
@@ -476,18 +495,14 @@ class EditorWindow:
         self.notebook.set_size_request(appconsts.NOTEBOOK_WIDTH, appconsts.TOP_ROW_HEIGHT)
         media_label = Gtk.Label(label=_("Media"))
         media_label.no_dark_bg = True
-        self.notebook.append_page(mm_panel, media_label)
+        if editorpersistance.prefs.global_layout == appconsts.SINGLE_WINDOW:
+            self.notebook.append_page(mm_panel, media_label)
         self.notebook.append_page(media_log_panel, Gtk.Label(label=_("Range Log")))
         self.notebook.append_page(self.effects_panel, Gtk.Label(label=_("Filters")))
         self.notebook.append_page(self.compositors_panel, Gtk.Label(label=_("Compositors")))
         self.notebook.append_page(project_panel, Gtk.Label(label=_("Project")))
         self.notebook.append_page(render_panel, Gtk.Label(label=_("Render")))
         self.notebook.set_tab_pos(Gtk.PositionType.BOTTOM)
-        #self.notebook.set_show_tabs(False) 
-
-        # Right notebook, used for Widescreen and Two row layouts
-        self.right_notebook = Gtk.Notebook()
-        self.right_notebook.set_tab_pos(Gtk.PositionType.BOTTOM)
 
         # Position bar and decorative frame  for it
         self.pos_bar = PositionBar()
@@ -502,39 +517,42 @@ class EditorWindow:
         self._create_monitor_row_widgets()
         self.player_buttons = glassbuttons.PlayerButtons()
         self.player_buttons.widget.set_tooltip_text(_("Prev Frame - Arrow Left\nNext Frame - Arrow Right\nPlay - Space\nStop - Space\nMark In - I\nMark Out - O\nClear Marks\nTo Mark In\nTo Mark Out"))
-        self.monitor_source.modify_font(Pango.FontDescription("sans bold 8"))
         player_buttons_row = Gtk.HBox(False, 0)
-
         player_buttons_row.pack_start(self.player_buttons.widget, False, True, 0)
-        player_buttons_row.pack_start(self.monitor_source, True, True, 0)
+        player_buttons_row.pack_start(pos_bar_frame, True, True, 0)
         player_buttons_row.set_margin_bottom(2)
 
         # Creates monitor switch buttons
         self._create_monitor_buttons()
 
+        # Monitor top info row
+        monitor_info_row = Gtk.HBox(False, 1)
+        monitor_info_row.pack_start(self.monitor_source, False, False, 0)
+        monitor_info_row.pack_start(Gtk.Label(), True, False, 0)
+        monitor_info_row.pack_start(self.info1, False, False, 0)
+
         # Switch / pos bar row
         self.view_mode_select = guicomponents.get_monitor_view_select_combo(lambda w, e: tlineaction.view_mode_menu_lauched(w, e))
+        self.trim_view_select = guicomponents.get_trim_view_select_combo(lambda w, e: monitorevent.trim_view_menu_launched(w, e))
         sw_pos_hbox = Gtk.HBox(False, 1)
-        sw_pos_hbox.pack_start(self.sequence_editor_b, False, True, 0)
-        sw_pos_hbox.pack_start(self.clip_editor_b, False, True, 0)
-        sw_pos_hbox.pack_start(pos_bar_frame, True, True, 0)
+        sw_pos_hbox.pack_start(self.sequence_editor_b, True, True, 0)
+        sw_pos_hbox.pack_start(self.clip_editor_b, True, True, 0)
+        sw_pos_hbox.pack_start(self.trim_view_select.widget, False, False, 0)
         sw_pos_hbox.pack_start(self.view_mode_select.widget, False, False, 0)
         sw_pos_hbox.set_margin_top(4)
         sw_pos_hbox.set_margin_left(2)
         
         # Video display
-        black_box = Gtk.EventBox()
-        black_box.add(Gtk.Label())
-        bg_color = Gdk.Color(red=0.0, green=0.0, blue=0.0)
-        black_box.modify_bg(Gtk.StateType.NORMAL, bg_color)
-        
-        self.tline_display = black_box # This could be any GTK+ widget (that is not "windowless"), only its XWindow draw rect 
-                                       # is used to position and scale SDL overlay that actually displays video.
+        monitor_widget = monitorwidget.MonitorWidget()
+        self.tline_display = monitor_widget.get_monitor()
+        self.monitor_widget = monitor_widget
+
         dnd.connect_video_monitor(self.tline_display)
 
         # Monitor
         monitor_vbox = Gtk.VBox(False, 1)
-        monitor_vbox.pack_start(self.tline_display, True, True, 0)
+        monitor_vbox.pack_start(monitor_info_row, False, True, 0)
+        monitor_vbox.pack_start(monitor_widget.widget, True, True, 0)
         monitor_vbox.pack_start(sw_pos_hbox, False, True, 0)
         monitor_vbox.pack_start(player_buttons_row, False, True, 0)
 
@@ -552,9 +570,13 @@ class EditorWindow:
 
         # Top row paned
         self.top_paned = Gtk.HPaned()
-        self.top_paned.pack1(notebook_vbox, resize=False, shrink=False)
-        self.top_paned.pack2(monitor_frame, resize=True, shrink=False)
-
+        if editorpersistance.prefs.global_layout == appconsts.SINGLE_WINDOW:
+            self.top_paned.pack1(notebook_vbox, resize=False, shrink=False)
+            self.top_paned.pack2(monitor_frame, resize=True, shrink=False)
+        else:
+            self.top_paned.pack1(mm_panel, resize=False, shrink=False)
+            self.top_paned.pack2(notebook_vbox, resize=True, shrink=False)
+            
         # Top row
         self.top_row_hbox = Gtk.HBox(False, 0)
         self.top_row_hbox.pack_start(self.top_paned, True, True, 0)
@@ -681,7 +703,6 @@ class EditorWindow:
         if w != 0: # non-existing prefs file causes w and h to be 0
             if (float(w) / editorstate.SCREEN_WIDTH > 0.95) and (float(h) / editorstate.SCREEN_HEIGHT > 0.95):
                 self.window.maximize()
-                print "maximize" 
             else:
                 self.window.resize(w, h)
                 self.window.set_position(Gtk.WindowPosition.CENTER)
@@ -690,7 +711,28 @@ class EditorWindow:
                 
         # Show window and all of its components
         self.window.show_all()
-        
+
+        # Show Monitor Window in two window mode
+        if editorpersistance.prefs.global_layout != appconsts.SINGLE_WINDOW:
+            pane2 = Gtk.VBox(False, 1)
+            pane2.pack_start(monitor_frame, True, True, 0)
+            
+            # Set pane and show window
+            self.window2.add(pane2)
+            self.window2.set_title("Flowblade")
+
+            # Maximize if it seems that we exited maximized, else set size
+            w, h, x, y = editorpersistance.prefs.exit_allocation_window_2
+                
+            if w != 0: # non-existing prefs file causes w and h to be 0
+                if (float(w) / editorstate.SCREEN_WIDTH > 0.95) and (float(h) / editorstate.SCREEN_HEIGHT > 0.95):
+                    self.window2.maximize()
+                else:
+                    self.window2.resize(w, h)
+            
+            self.window2.move(x, y)
+            self.window2.show_all()
+                        
         # Set paned positions
         self.mm_paned.set_position(editorpersistance.prefs.mm_paned_position)
         self.top_paned.set_position(editorpersistance.prefs.top_paned_position)
@@ -707,7 +749,7 @@ class EditorWindow:
         mb_menu = Gtk.Menu()
         tc_left = Gtk.RadioMenuItem()
         tc_left.set_label(_("Timecode Left").encode('utf-8'))
-        tc_left.set_active(True)
+        #tc_left.set_active(appconsts)
         tc_left.connect("activate", lambda w: middlebar._show_buttons_TC_LEFT_layout(w))
         mb_menu.append(tc_left)
 
@@ -715,7 +757,13 @@ class EditorWindow:
         tc_middle.connect("activate", lambda w: middlebar._show_buttons_TC_MIDDLE_layout(w))
         mb_menu.append(tc_middle)
 
-        if editorpersistance.prefs.midbar_tc_left == True:
+        components_centered = Gtk.RadioMenuItem.new_with_label([tc_left], _("Components Centered").encode('utf-8'))
+        components_centered.connect("activate", lambda w: middlebar._show_buttons_COMPONETS_CENTERED_layout(w))
+        mb_menu.append(components_centered)
+
+        if editorpersistance.prefs.midbar_layout == appconsts.MIDBAR_COMPONENTS_CENTERED:
+            components_centered.set_active(True)
+        elif editorpersistance.prefs.midbar_layout == appconsts.MIDBAR_TC_LEFT:
             tc_left.set_active(True)
         else:
             tc_middle.set_active(True)
@@ -745,20 +793,18 @@ class EditorWindow:
         sep = Gtk.SeparatorMenuItem()
         menu.append(sep)
 
-        if not (editorstate.screen_size_small_height() == True or editorstate.screen_size_small_height() == True):
-            
-            show_monitor_info_item = Gtk.CheckMenuItem(_("Show Monitor Sequence Profile").encode('utf-8'))
-            show_monitor_info_item.set_active(editorpersistance.prefs.show_sequence_profile)
-            show_monitor_info_item.connect("toggled", lambda w: middlebar._show_monitor_info_toggled(w))
-            menu.append(show_monitor_info_item)
+        show_monitor_info_item = Gtk.CheckMenuItem(_("Show Monitor Sequence Profile").encode('utf-8'))
+        show_monitor_info_item.set_active(editorpersistance.prefs.show_sequence_profile)
+        show_monitor_info_item.connect("toggled", lambda w: middlebar._show_monitor_info_toggled(w))
+        menu.append(show_monitor_info_item)
 
-            show_vu_item = Gtk.CheckMenuItem(_("Show Master Volume Meter").encode('utf-8'))
-            show_vu_item.set_active(editorpersistance.prefs.show_vu_meter)
-            show_vu_item.connect("toggled", lambda w: self._show_vu_meter(w))
-            menu.append(show_vu_item)
+        show_vu_item = Gtk.CheckMenuItem(_("Show Master Volume Meter").encode('utf-8'))
+        show_vu_item.set_active(editorpersistance.prefs.show_vu_meter)
+        show_vu_item.connect("toggled", lambda w: self._show_vu_meter(w))
+        menu.append(show_vu_item)
 
-            sep = Gtk.SeparatorMenuItem()
-            menu.append(sep)
+        sep = Gtk.SeparatorMenuItem()
+        menu.append(sep)
 
         interp_menu_item = Gtk.MenuItem(_("Monitor Playback Interpolation").encode('utf-8'))
         interp_menu = Gtk.Menu()
@@ -800,10 +846,8 @@ class EditorWindow:
     def _init_gui_to_prefs(self):
         if editorpersistance.prefs.tabs_on_top == True:
             self.notebook.set_tab_pos(Gtk.PositionType.TOP)
-            self.right_notebook.set_tab_pos(Gtk.PositionType.TOP)
         else:
             self.notebook.set_tab_pos(Gtk.PositionType.BOTTOM)
-            self.right_notebook.set_tab_pos(Gtk.PositionType.BOTTOM)
 
     def _show_tabs_up(self, widget):
         if widget.get_active() == False:
@@ -826,7 +870,7 @@ class EditorWindow:
         self._update_top_row(True)
 
     def _update_top_row(self, show_all=False):
-        if editorpersistance.prefs.show_vu_meter and editorstate.screen_size_small_height() == False:
+        if editorpersistance.prefs.show_vu_meter:
             if len(self.top_row_hbox) == 1:
                 self.top_row_hbox.pack_end(audiomonitoring.get_master_meter(), False, False, 0)
         else:
@@ -846,7 +890,6 @@ class EditorWindow:
         self.sequence_editor_b.connect("clicked", 
                         lambda w,e: self._monitor_switch_handler(w), 
                         None)
-        self.sequence_editor_b.set_size_request(100, 25)
 
         self.clip_editor_b = Gtk.RadioButton.new_from_widget(self.sequence_editor_b)#,_("Clip"))
         self.clip_editor_b.set_mode(False)
@@ -854,7 +897,6 @@ class EditorWindow:
         self.clip_editor_b.connect("clicked",
                         lambda w,e: self._monitor_switch_handler(w),
                         None)
-        self.clip_editor_b.set_size_request(100, 25)
 
     def _monitor_switch_handler(self, widget):
         # We get two "clicked" events per toggle, send through only the one
@@ -867,21 +909,29 @@ class EditorWindow:
             and (widget == self.clip_editor_b)):
             updater.display_clip_in_monitor()
 
-    def layouts_pressed(self, widget, event):
-        print "kkkkkkkkkkkkkkkkkkk"
-
     def connect_player(self, mltplayer):
         # Buttons
         # NOTE: ORDER OF CALLBACKS IS THE SAME AS ORDER OF BUTTONS FROM LEFT TO RIGHT
-        pressed_callback_funcs = [monitorevent.prev_pressed,
-                                  monitorevent.next_pressed,
-                                  monitorevent.play_pressed,
-                                  monitorevent.stop_pressed,
-                                  monitorevent.mark_in_pressed,
-                                  monitorevent.mark_out_pressed,
-                                  monitorevent.marks_clear_pressed,
-                                  monitorevent.to_mark_in_pressed,
-                                  monitorevent.to_mark_out_pressed]
+        # Jul-2016 - SvdB - For play/pause button
+        if editorpersistance.prefs.play_pause == False:
+            pressed_callback_funcs = [monitorevent.prev_pressed,
+                                      monitorevent.next_pressed,
+                                      monitorevent.play_pressed,
+                                      monitorevent.stop_pressed,
+                                      monitorevent.mark_in_pressed,
+                                      monitorevent.mark_out_pressed,
+                                      monitorevent.marks_clear_pressed,
+                                      monitorevent.to_mark_in_pressed,
+                                      monitorevent.to_mark_out_pressed]
+        else:
+            pressed_callback_funcs = [monitorevent.prev_pressed,
+                                      monitorevent.next_pressed,
+                                      monitorevent.play_pressed,
+                                      monitorevent.mark_in_pressed,
+                                      monitorevent.mark_out_pressed,
+                                      monitorevent.marks_clear_pressed,
+                                      monitorevent.to_mark_in_pressed,
+                                      monitorevent.to_mark_out_pressed]
         self.player_buttons.set_callbacks(pressed_callback_funcs)
 
         # Monitor position bar
@@ -892,7 +942,9 @@ class EditorWindow:
         middlebar.create_edit_buttons_row_buttons(self, modes_pixbufs)
     
         buttons_row = Gtk.HBox(False, 1)
-        if editorpersistance.prefs.midbar_tc_left == True:
+        if editorpersistance.prefs.midbar_layout == appconsts.MIDBAR_COMPONENTS_CENTERED:
+            middlebar.fill_with_COMPONETS_CENTERED_pattern(buttons_row, self)
+        elif editorpersistance.prefs.midbar_layout == appconsts.MIDBAR_TC_LEFT:
             middlebar.fill_with_TC_LEFT_pattern(buttons_row, self)
         else:
             middlebar.fill_with_TC_MIDDLE_pattern(buttons_row, self)
@@ -1044,8 +1096,10 @@ class EditorWindow:
         self.tc = guicomponents.MonitorTCDisplay()
         self.monitor_source = Gtk.Label(label="sequence1")
         self.monitor_source.set_ellipsize(Pango.EllipsizeMode.END)
-
-
+        self.monitor_source.modify_font(Pango.FontDescription("sans bold 8"))
+        self.info1 = Gtk.Label(label="--:--:--:--")
+        self.info1.set_ellipsize(Pango.EllipsizeMode.END)
+        self.info1.modify_font(Pango.FontDescription("sans bold 8"))
 
 def _this_is_not_used():
     print "THIS WAS USED!!!!!"
