@@ -42,6 +42,8 @@ import menuactions
 import monitorevent
 import mltrefhold
 import tlineaction
+import tlinewidgets
+import trimmodes
 import updater
 import projectaction
 
@@ -115,6 +117,7 @@ def key_down(widget, event):
         if PLAYER().is_playing():
             monitorevent.stop_pressed()
         PLAYER().seek_frame(0)
+        _move_to_beginning()
         return True
 
     # Select all with CTRL + A in media panel
@@ -145,7 +148,7 @@ def _timeline_has_focus():
        or gui.tline_scale.widget.is_focus()
        or glassbuttons.focus_group_has_focus(glassbuttons.DEFAULT_FOCUS_GROUP)):
         return True
-
+    
     return False
     
 def _handle_tline_key_event(event):
@@ -236,6 +239,23 @@ def _handle_tline_key_event(event):
         medialog.log_range_clicked()
         return True
 
+
+    # Key bindings for keyboard trimming
+    if editorstate.current_is_active_trim_mode() == True:
+        # LEFT ARROW, prev frame
+        if event.keyval == Gdk.KEY_Left:
+            trimmodes.left_arrow_pressed((event.get_state() & Gdk.ModifierType.CONTROL_MASK))
+            return True
+
+        # RIGHT ARROW, next frame
+        if event.keyval == Gdk.KEY_Right:
+            trimmodes.right_arrow_pressed((event.get_state() & Gdk.ModifierType.CONTROL_MASK))
+            return True
+
+        if event.keyval == Gdk.KEY_Return:
+            trimmodes.enter_pressed()
+            return True
+
     # Key bindings for MOVE MODES and _NO_EDIT modes
     if editorstate.current_is_move_mode() or editorstate.current_is_active_trim_mode() == False:
          # UP ARROW, next cut
@@ -311,7 +331,11 @@ def _handle_tline_key_event(event):
                 monitorevent.l_pressed()
             return True
 
-
+        # R
+        if event.keyval == Gdk.KEY_r:
+            tlineaction.resync_button_pressed()
+            return True
+            
         # DELETE
         if event.keyval == Gdk.KEY_Delete:
             # Clip selection and compositor selection are mutually exclusive, 
@@ -324,6 +348,7 @@ def _handle_tline_key_event(event):
             if PLAYER().is_playing():
                 monitorevent.stop_pressed()
             PLAYER().seek_frame(0)
+            _move_to_beginning()
             return True
     else:
         # HOME
@@ -333,6 +358,7 @@ def _handle_tline_key_event(event):
             gui.editor_window.handle_insert_move_mode_button_press()
             gui.editor_window.set_mode_selector_to_mode()
             PLAYER().seek_frame(0)
+            _move_to_beginning()
             return True
 
     return False
@@ -579,3 +605,11 @@ def _get_focus_keyframe_editor(keyframe_editor_widgets):
         if kfeditor.get_focus_child() != None:
            return kfeditor
     return None
+
+def _move_to_beginning():
+    tlinewidgets.pos = 0
+    updater.repaint_tline()
+    updater.update_tline_scrollbar()
+    
+    
+    
