@@ -878,6 +878,12 @@ class AbstractScreenEditor:
         delta_x = x - self.mouse_start_x
         delta_y = y - self.mouse_start_y
         
+        if state & Gdk.ModifierType.SHIFT_MASK:
+            if abs(x - self.mouse_start_x) < abs(y - self.mouse_start_y):
+                delta_x = 0
+            else:
+                delta_y = 0
+        
         self._shape__motion_notify_event(delta_x, delta_y)
 
         self.parent_editor.queue_draw()
@@ -893,6 +899,12 @@ class AbstractScreenEditor:
         delta_x = event.x - self.mouse_start_x
         delta_y = event.y - self.mouse_start_y
 
+        if event.get_state() & Gdk.ModifierType.SHIFT_MASK:
+            if abs(event.x - self.mouse_start_x) < abs(event.y - self.mouse_start_y):
+                delta_x = 0
+            else:
+                delta_y = 0
+                
         self._shape_release_event(delta_x, delta_y)
             
         self.parent_editor.geometry_edit_finished()
@@ -1721,7 +1733,8 @@ class KeyFrameEditor(AbstractKeyFrameEditor):
     def prev_pressed(self):
         self.clip_editor.set_prev_active()
         self.update_editor_view()
-    
+        self.buttons_row.set_kf_info(self.clip_editor.get_kf_info())
+
     def prev_frame_pressed(self):
         self.clip_editor.move_clip_frame(-1)
         self.update_editor_view()
@@ -1867,7 +1880,8 @@ class GeometryEditor(AbstractKeyFrameEditor):
         self.geom_kf_edit.set_keyframe_to_edit_shape(self.clip_editor.active_kf_index)
         self.update_editor_view_with_frame(self.clip_editor.current_clip_frame)
         self.update_property_value()
-    
+        self.buttons_row.set_kf_info(self.clip_editor.get_kf_info())
+
     def arrow_edit(self, keyval):
         self.geom_kf_edit.handle_arrow_edit(keyval)
         self.geom_kf_edit.set_keyframe_to_edit_shape(self.clip_editor.active_kf_index)
