@@ -28,6 +28,7 @@ from gi.repository import Gtk, Gdk
 
 import appconsts
 import dialogutils
+import editorpersistance # Jul-2016 - SvdB - For play/pause button
 import dnd
 import edit
 from editorstate import current_sequence
@@ -66,7 +67,15 @@ edit_data = None
 # These four buttons act differently in trimmodes and move modes
 def play_pressed():
     # This handles only move modes, see trimmodes.py module for others.
-    PLAYER().start_playback()  
+    # Jul-2016 - SvdB - Added code to handle play/pause button
+    if editorpersistance.prefs.play_pause == True:
+        if PLAYER().is_playing():
+            PLAYER().stop_playback()
+        else:
+            PLAYER().start_playback()
+    else:
+        # Original code
+        PLAYER().start_playback()  
 
 def stop_pressed():
     # This handles only move modes, see trimmodes.py module for others.
@@ -413,6 +422,7 @@ def _move_mode_pressed(event, frame):
         return
         
     # Check locking for pressed track
+    # This ain't used crrently!!!!!
     if _track_is_locked(track):
         clear_selected_clips()
         pressed_on_selected = False
@@ -432,6 +442,7 @@ def _move_mode_pressed(event, frame):
             # seems to a single blank area. All of these must be
             # selected together automatically or user will be exposed to
             # this impl. detail unnecesserarely.
+            # THIS HAS BEEN CHANGED BECAUSE BLANKS ARE NOW AUTO CONSOLIDATED AFTER ALL EDITS.
             range_in, range_out = _get_blanck_range(track, clip_index)
             _select_multiple_clips(track.id, range_in, range_out)
             pressed_on_selected = False
@@ -494,9 +505,9 @@ def _move_mode_pressed(event, frame):
                  "track_object":track,
                  "to_track_object":track,
                  "move_on":False,
-                 "press_frame":frame,
+                 "press_frame":frame, # on timeline at mouse press
                  "current_frame":frame,
-                 "first_clip_start":insert_frame,
+                 "first_clip_start":insert_frame,  # on timeline at mouse press
                  "insert_frame":insert_frame,
                  "clip_lengths":clip_lengths,
                  "mouse_start_x":x,
