@@ -26,9 +26,11 @@ Load, save, add media file, etc...
 import datetime
 import glob
 import md5
+import mlt
 import os
 from os import listdir
 from os.path import isfile, join
+from PIL import Image
 import re
 import shutil
 import time
@@ -73,7 +75,6 @@ import utils
 save_time = None
 save_icon_remove_event_id = None
 
-
 #--------------------------------------- worker threads
 class LoadThread(threading.Thread):
     
@@ -98,6 +99,7 @@ class LoadThread(threading.Thread):
             editorstate.project_is_loading = True
             
             project = persistance.load_project(self.filename)
+
             sequence.set_track_counts(project)
             
             editorstate.project_is_loading = False
@@ -168,14 +170,14 @@ class LoadThread(threading.Thread):
         Gdk.threads_leave()
         ticker.stop_ticker()
 
-
     def _missing_file_dialog_callback(self, dialog, response_id):
         if self.open_check.get_active() == True:
             medialinker.display_linker(self.filename)
             dialog.destroy()
         else:
             dialog.destroy()
-    
+
+
 class AddMediaFilesThread(threading.Thread):
     
     def __init__(self, filenames):
@@ -188,6 +190,7 @@ class AddMediaFilesThread(threading.Thread):
         gui.editor_window.window.get_window().set_cursor(watch)
         Gdk.threads_leave()
 
+        is_first_video_load = PROJECT().is_first_video_load()
         duplicates = []
         succes_new_file = None
         filenames = self.filenames
@@ -225,9 +228,50 @@ class AddMediaFilesThread(threading.Thread):
 
         if len(duplicates) > 0:
             GObject.timeout_add(10, _duplicates_info, duplicates)
-        
+
+        if is_first_video_load:
+            GObject.timeout_add(10, _first_load_profile_check)
+            
         audiowaveformrenderer.launch_audio_levels_rendering(filenames)
 
+
+class UpdateMediaLengthsThread(threading.Thread):
+    
+    def __init__(self):
+        threading.Thread.__init__(self)
+
+    def run(self):
+        print "Updating media lengths:"
+
+        Gdk.threads_enter()
+        dialog = dialogs.update_media_lengths_progress_dialog()
+        time.sleep(0.1)
+        Gdk.threads_leave()
+
+        for key, media_file in PROJECT().media_files.iteritems():
+            print media_file.name
+            if media_file.type == appconsts.VIDEO or media_file.type == appconsts.IMAGE_SEQUENCE:
+                print media_file.name
+                Gdk.threads_enter()
+                dialog.info.set_text(media_file.name)
+                Gdk.threads_leave()
+        
+                producer = mlt.Producer(PROJECT().profile, str(media_file.path))
+                if producer.is_valid() == False:
+                    print "not valid producer"
+                    continue
+
+                length = producer.get_length()
+                media_file.length = length
+                
+        PROJECT().update_media_lengths_on_load = False
+        
+        Gdk.threads_enter()
+        dialog.destroy()
+        Gdk.threads_leave()
+        
+        print "Updating media lengths done."
+        
 def _duplicates_info(duplicates):
     primary_txt = _("Media files already present in project were opened!")
     MAX_DISPLAYED_ITEMS = 3
@@ -247,6 +291,28 @@ def _duplicates_info(duplicates):
     dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
     return False
 
+def _first_load_profile_check():
+    for uid, media_file in PROJECT().media_files.iteritems():
+        if media_file.type == appconsts.VIDEO:
+            if media_file.matches_project_profile() == False:
+                dialogs.not_matching_media_info_dialog(PROJECT(), media_file, _not_matching_media_info_callback)
+                break
+
+def _not_matching_media_info_callback(dialog, response_id, media_file):
+    dialog.destroy()
+            
+    if response_id == Gtk.ResponseType.ACCEPT:
+        # Save in hidden and open
+        match_profile_index = mltprofiles.get_closest_matching_profile_index(media_file.info)
+        profile = mltprofiles.get_profile_for_index(match_profile_index)
+
+        path = utils.get_hidden_user_dir_path() + "/" + PROJECT().name
+        PROJECT().update_media_lengths_on_load = True
+        
+        persistance.save_project(PROJECT(), path, profile.description()) #<----- HERE
+        
+        actually_load_project(path)
+
 def _load_pulse_bar():
     Gdk.threads_enter()
     try: 
@@ -324,10 +390,19 @@ def save_project():
 def _save_project_in_last_saved_path():
     updater.set_info_icon(Gtk.STOCK_SAVE)
 
+    try:
+        
+        persistance.save_project(PROJECT(), PROJECT().last_save_path) #<----- HERE
+        
+    except IOError as ioe:
+        updater.set_info_icon(None)
+        primary_txt = "I/O error({0})".format(ioe.errno)
+        secondary_txt = ioe.strerror + "."
+        dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window, is_info=False)
+        return
+
     PROJECT().events.append(projectdata.ProjectEvent(projectdata.EVENT_SAVED, PROJECT().last_save_path))
-
-    persistance.save_project(PROJECT(), PROJECT().last_save_path) #<----- HERE
-
+    
     global save_icon_remove_event_id
     save_icon_remove_event_id = GObject.timeout_add(500, remove_save_icon)
 
@@ -350,6 +425,18 @@ def _save_as_dialog_callback(dialog, response_id):
         PROJECT().name = unicode(os.path.basename(filenames[0]), "utf-8")
         updater.set_info_icon(Gtk.STOCK_SAVE)
 
+        try:
+            
+            persistance.save_project(PROJECT(), PROJECT().last_save_path) #<----- HERE
+            
+        except IOError as ioe:
+            dialog.destroy()
+            updater.set_info_icon(None)
+            primary_txt = "I/O error({0})".format(ioe.errno)
+            secondary_txt = ioe.strerror + "."
+            dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window, is_info=False)
+            return
+
         if len(PROJECT().events) == 0: # Save as... with 0 project events is considered Project creation
             p_event = projectdata.ProjectEvent(projectdata.EVENT_CREATED_BY_SAVING, PROJECT().last_save_path)
             PROJECT().events.append(p_event)
@@ -357,8 +444,6 @@ def _save_as_dialog_callback(dialog, response_id):
             p_event = projectdata.ProjectEvent(projectdata.EVENT_SAVED_AS, (PROJECT().name, PROJECT().last_save_path))
             PROJECT().events.append(p_event)
             
-        persistance.save_project(PROJECT(), PROJECT().last_save_path) #<----- HERE
-        
         app.stop_autosave()
         app.start_autosave()
         
@@ -411,6 +496,10 @@ def _do_snapshot_save(root_folder_path, project_name):
     save_thread = SnaphotSaveThread(root_folder_path, project_name)
     save_thread.start()
 
+def update_media_lengths():
+    update_thread = UpdateMediaLengthsThread()
+    update_thread.start()
+    
 def change_project_profile():
     dialogs.change_profile_project_dialog(PROJECT(), _change_project_profile_callback)
 
@@ -421,12 +510,37 @@ def _change_project_profile_callback(dialog, response_id, profile_combo, out_fol
         profile = mltprofiles.get_profile_for_index(profile_combo.get_active())
         path = folder + "/" + name
 
+        PROJECT().update_media_lengths_on_load = True # saved version needs to do this
+        
         persistance.save_project(PROJECT(), path, profile.description()) #<----- HERE
 
+        PROJECT().update_media_lengths_on_load = False
+
         dialog.destroy()
     else:
         dialog.destroy()
+        
+""" Feature disabled, maybe reactivated later
+def change_profile_to_match_media(media_file):
+    dialogs.change_profile_project_to_match_media_dialog(PROJECT(), media_file, _change_project_profile_to_match_media_callback)
 
+def _change_project_profile_to_match_media_callback(dialog, response_id, match_profile_index, out_folder, project_name_entry):
+    if response_id == Gtk.ResponseType.ACCEPT:
+        folder = "/" + out_folder.get_uri().lstrip("file:/")
+        name = project_name_entry.get_text()
+        profile = mltprofiles.get_profile_for_index(match_profile_index)
+        path = folder + "/" + name
+        
+        PROJECT().update_media_lengths_on_load = True # saved version needs to do this
+        
+        persistance.save_project(PROJECT(), path, profile.description()) #<----- HERE
+
+        PROJECT().update_media_lengths_on_load = False
+
+        dialog.destroy()
+    else:
+        dialog.destroy()
+"""
 
 class SnaphotSaveThread(threading.Thread):
     
@@ -557,6 +671,38 @@ def remove_save_icon():
 
 def open_recent_project(widget, index):
     path = editorpersistance.recent_projects.projects[index]
+    if _project_empty() == True:
+        _actually_open_recent(path)
+    else:
+        dialogs.exit_confirm_dialog(_open_recent_shutdown_dialog_callback, get_save_time_msg(), gui.editor_window.window, editorstate.PROJECT().name, path)
+
+def _project_empty():
+    for seq in PROJECT().sequences:
+        if not seq.is_empty():
+            return False
+    
+    return True
+    
+def _open_recent_shutdown_dialog_callback(dialog, response_id, path):
+    dialog.destroy()
+    
+    # Handle poroject close responses
+    if response_id == Gtk.ResponseType.CLOSE:# "Don't Save"
+        pass
+    elif response_id ==  Gtk.ResponseType.YES:# "Save"
+        if editorstate.PROJECT().last_save_path != None:
+            persistance.save_project(editorstate.PROJECT(), editorstate.PROJECT().last_save_path)
+        else:
+            dialogutils.warning_message(_("Project has not been saved previously"), 
+                                    _("Save project with File -> Save As before closing."),
+                                    gui.editor_window.window)
+            return
+    else: # "Cancel"
+        return
+    
+    _actually_open_recent(path)
+
+def _actually_open_recent(path):
     if not os.path.exists(path):
         editorpersistance.recent_projects.projects.pop(index)
         editorpersistance.fill_recents_menu_widget(gui.editor_window.uimanager.get_widget('/MenuBar/FileMenu/OpenRecent'), open_recent_project)
@@ -567,10 +713,25 @@ def open_recent_project(widget, index):
 
     actually_load_project(path)
 
+def get_save_time_msg():
+    if save_time == None:
+        return _("Project has not been saved since it was opened.")
+    
+    save_ago = (time.clock() - save_time) / 60.0
+
+    if save_ago < 1:
+        return _("Project was saved less than a minute ago.")
+
+    if save_ago < 2:
+        return _("Project was saved one minute ago.")
+    
+    return _("Project was saved ") + str(int(save_ago)) + _(" minutes ago.")
+    
 # ---------------------------------- rendering
 def do_rendering():
-    _write_out_render_item(True)
-    batchrendering.launch_single_rendering()
+    success = _write_out_render_item(True)
+    if success:
+        batchrendering.launch_single_rendering()
 
 def add_to_render_queue():
     _write_out_render_item(False)
@@ -592,7 +753,7 @@ def _write_out_render_item(single_render_item_item):
     if start_frame == -1 or end_frame == -1:
         if render.widgets.range_cb.get_active() == 1:
             rendergui.no_good_rander_range_info()
-            return
+            return False
 
     # Create batchrendering.RenderData object.
     # batchrendering.RenderData object is only used to display info about render,
@@ -621,7 +782,7 @@ def _write_out_render_item(single_render_item_item):
             primary_txt = _("Render launch failed!")
             secondary_txt = _("Error message: ") + str(e)
             dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window, is_info=False)
-            return
+            return False
     else: # batch render item
         # Add item
         try:
@@ -635,9 +796,9 @@ def _write_out_render_item(single_render_item_item):
             primary_txt = _("Adding item to render queue failed!")
             secondary_txt = _("Error message: ") + str(e)
             dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window, is_info=False)
-            return
-
+            return False
 
+    return True
 
 # ----------------------------------- media files
 def add_media_files(this_call_is_retry=False):
@@ -843,6 +1004,10 @@ def _display_file_info(media_file):
 
     width = info["width"]
     height = info["height"]
+    if media_file.type == appconsts.IMAGE:
+        graphic_img = Image.open(media_file.path)
+        width, height = graphic_img.size 
+
     size = str(width) + " x " + str(height)
     length = utils.get_tc_string(info["length"])
 
@@ -854,17 +1019,36 @@ def _display_file_info(media_file):
     vcodec = info["vcodec"]
     acodec = info["acodec"]
     
+    if vcodec == None:
+        vcodec = _("N/A")
+    if acodec == None:
+        acodec = _("N/A")
+
     channels = str(info["channels"]) 
     frequency =  str(info["frequency"]) + "Hz"
+
+    if media_file.type == appconsts.VIDEO:
+        match_profile_index = mltprofiles.get_closest_matching_profile_index(info)
+        match_profile_name =  mltprofiles.get_profile_name_for_index(match_profile_index)
+    else:
+        match_profile_name = _("N/A")
+    
+    if media_file.type == appconsts.VIDEO:
+        if media_file.matches_project_profile():
+            matches_project_profile = _("Yes")
+        else:
+            matches_project_profile = _("No")
+    else:
+        matches_project_profile = _("N/A")
         
     try:
         num = info["fps_num"]
         den = info["fps_den"]
         fps = float(num/den) 
     except:
-        fps ="N/A"
+        fps = _("N/A")
     
-    dialogs.file_properties_dialog((media_file, img, size, length, vcodec, acodec, channels, frequency, fps))
+    dialogs.file_properties_dialog((media_file, img, size, length, vcodec, acodec, channels, frequency, fps, match_profile_name, matches_project_profile))
 
 def remove_unused_media():
     # Create path -> media item dict
@@ -1163,6 +1347,8 @@ def media_file_menu_item_selected(widget, data):
         delete_media_files()
     if item_id == "Render Proxy File":
         proxyediting.create_proxy_menu_item_selected(media_file)
+    #if item_id == "Project Profile":
+    #    change_profile_to_match_media(media_file)
 
 def _select_treeview_on_pos_and_return_row_and_column_title(event, treeview):
     selection = treeview.get_selection()
