@@ -35,6 +35,7 @@ from gi.repository import GdkPixbuf
 import appconsts
 import editorpersistance
 from editorstate import PLAYER
+from editorstate import PROJECT
 import mltprofiles
 import mltrefhold
 import patternproducer
@@ -83,6 +84,9 @@ class Project:
         self.media_log = []
         self.media_log_groups = []
         self.proxy_data = miscdataobjects.ProjectProxyEditingData()
+        self.update_media_lengths_on_load = False # old projects < 1.10 had wrong media length data which just was never used.
+                                                  # 1.10 needed that data for the first time and required recreating it correctly for older projects
+
         self.SAVEFILE_VERSION = SAVEFILE_VERSION
         
         # c_seq is the currently edited Sequence
@@ -97,9 +101,8 @@ class Project:
     
     def init_thumbnailer(self):
         global thumbnailer
-        if thumbnailer == None:
-            thumbnailer = Thumbnailer()
-            thumbnailer.set_context(self.profile)
+        thumbnailer = Thumbnailer()
+        thumbnailer.set_context(self.profile)
 
     def add_image_sequence_media_object(self, resource_path, name, length):
         media_object = self.add_media_file(resource_path)
@@ -120,12 +123,13 @@ class Project:
         if media_type == appconsts.AUDIO:
             icon_path = respaths.IMAGE_PATH + "audio_file.png"
             length = thumbnailer.get_file_length(file_path)
+            info = None
         else: # For non-audio we need write a thumbbnail file and get file lengh while we're at it
-             (icon_path, length) = thumbnailer.write_image(file_path)
+             (icon_path, length, info) = thumbnailer.write_image(file_path)
 
-          # Create media file object
+        # Create media file object
         media_object = MediaFile(self.next_media_file_id, file_path, 
-                               file_name, media_type, length, icon_path)
+                               file_name, media_type, length, icon_path, info)
 
         self._add_media_object(media_object)
         
@@ -261,12 +265,19 @@ class Project:
 
         return os.path.dirname(last_render_event.data)
 
+    def is_first_video_load(self):
+        for uid, media_file in self.media_files.iteritems():
+            if media_file.type == appconsts.VIDEO:
+                return False
         
+        return True
+
+
 class MediaFile:
     """
     Media file that can added to and edited in Sequence.
     """
-    def __init__(self, id, file_path, name, media_type, length, icon_path):
+    def __init__(self, id, file_path, name, media_type, length, icon_path, info):
         self.id = id
         self.path = file_path
         self.name = name
@@ -285,6 +296,8 @@ class MediaFile:
         
         self.current_frame = 0
 
+        self.info = info
+
         # Set default length for graphics files
         (f_name, ext) = os.path.splitext(self.name)
         if utils.file_extension_is_graphics_file(ext):
@@ -305,7 +318,7 @@ class MediaFile:
         icon = cairo.ImageSurface.create_from_png(self.icon_path)
         scaled_icon = cairo.ImageSurface(cairo.FORMAT_ARGB32, appconsts.THUMB_WIDTH, appconsts.THUMB_HEIGHT)
         cr = cairo.Context(scaled_icon)
-        cr.scale( float(appconsts.THUMB_WIDTH) / float(icon.get_width()), float(appconsts.THUMB_HEIGHT) / float(icon.get_height()))
+        cr.scale(float(appconsts.THUMB_WIDTH) / float(icon.get_width()), float(appconsts.THUMB_HEIGHT) / float(icon.get_height()))
         cr.set_source_surface(icon, 0, 0)
         cr.paint()
         
@@ -345,6 +358,23 @@ class MediaFile:
         self.path, self.second_file_path = self.second_file_path, self.path
         self.is_proxy_file = False
 
+    def matches_project_profile(self):
+        if (not hasattr(self, "info")): # to make really sure that old projects don't crash,
+            return True                            # but probably is not needed as attr is added at load
+        if self.info == None:
+            return True
+        
+        is_match = True # this is true for audio and graphics and image sequences and is only 
+                        # set false for video that does not match profile
+                        
+        if self.type == appconsts.VIDEO:
+            best_media_profile_index = mltprofiles.get_closest_matching_profile_index(self.info)
+            project_profile_index = mltprofiles.get_index_for_name(PROJECT().profile.description())
+            if best_media_profile_index != project_profile_index:
+                is_match = False
+        
+        return is_match
+        
 
 class BinColorClip:
     # DECPRECATED, this is replaced by patternproducer.BinColorClip.
@@ -415,6 +445,8 @@ class Thumbnailer:
         if producer.is_valid() == False:
             raise ProducerNotValidError(file_path)
 
+        info = utils.get_file_producer_info(producer)
+
         length = producer.get_length()
         frame = length / 2
         producer = producer.cut(frame, frame)
@@ -423,7 +455,7 @@ class Thumbnailer:
         consumer.connect(producer)
         consumer.run()
         
-        return (thumbnail_path, length)
+        return (thumbnail_path, length, info)
 
     def get_file_length(self, file_path):
         # This is used for audio files which don't need a thumbnail written
