@@ -36,6 +36,7 @@ import time
 import threading
 
 import dialogutils
+import editorstate
 from editorstate import current_sequence
 from editorstate import PROJECT
 from editorstate import PLAYER
@@ -95,122 +96,7 @@ class RenderLauncher(threading.Thread):
         PLAYER().set_render_callbacks(callbacks)
         PLAYER().start_rendering(self.render_consumer, self.start_frame, self.end_frame)
 
-"""
-def render_timeline():
-    if len(widgets.file_panel.movie_name.get_text()) == 0:
-        primary_txt = _("Render file name entry is empty")
-        secondary_txt = _("You have to provide a name for the file to be rendered.")
-        dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window)
-        return   
 
-    if os.path.exists(get_file_path()):
-        primary_txt = _("File: ") + get_file_path() + _(" already exists!")
-        secondary_txt = _("Do you want to overwrite existing file?")
-        dialogutils.warning_confirmation(_render_overwrite_confirm_callback, primary_txt, secondary_txt, gui.editor_window.window)
-    else:
-        _do_rendering()
-
-def _render_overwrite_confirm_callback(dialog, response_id):
-    dialog.destroy()
-    
-    if response_id == Gtk.ResponseType.ACCEPT:
-        _do_rendering()
-
-def _do_rendering():
-    print "timeline render..."
-    
-    global aborted
-    aborted = False
-    render_consumer = get_render_consumer()
-    if render_consumer == None:
-        return
-
-    # Set render start and end points
-    if widgets.range_cb.get_active() == 0:
-        start_frame = 0
-        end_frame = -1 # renders till finish
-    else:
-        start_frame = current_sequence().tractor.mark_in
-        end_frame = current_sequence().tractor.mark_out
-
-    # Only render a range if it is defined.
-    if start_frame == -1 or end_frame == -1:
-        if widgets.range_cb.get_active() == 1:
-            rendergui.no_good_rander_range_info()
-            return
-
-    file_path = get_file_path()
-    project_event = projectdata.ProjectEvent(projectdata.EVENT_RENDERED, file_path)
-    PROJECT().events.append(project_event)
-    projectinfogui.update_project_info()
-
-    # See if project and render fps match
-    cnum = render_consumer.profile().frame_rate_num()
-    cden = render_consumer.profile().frame_rate_den()
-    pnum = PROJECT().profile.frame_rate_num()
-    pden = PROJECT().profile.frame_rate_den()
-    
-    if (cnum == pnum) and (cden == pden):
-        frames_rates_match = True
-    else:
-        frames_rates_match = False
-
-    global progress_window
-    progress_window = rendergui.render_progress_dialog(_render_cancel_callback,
-                                                       gui.editor_window.window,
-                                                       frames_rates_match)
-                                                       
-    set_render_gui()
-
-    render_launch = RenderLauncher(render_consumer, start_frame, end_frame)
-    render_launch.start()
-
-def _render_cancel_callback(dialog, response_id):
-    global aborted
-    aborted = True
-    dialog.destroy()
-    PLAYER().consumer.stop()
-    PLAYER().producer.set_speed(0)
-"""
-
-# -------------------------------------------------- render consumer
-def get_render_consumer():
-    file_path = get_file_path()
-    if file_path == None:
-        return None
-
-    profile = get_current_profile()
-
-    if widgets.render_type_panel.type_combo.get_active() == 1: # Preset encodings
-        encoding_option = renderconsumer.non_user_encodings[widgets.render_type_panel.presets_selector.widget.get_active()]
-        if encoding_option.type != "img_seq":
-            consumer = renderconsumer.get_render_consumer_for_encoding(file_path,
-                                                                       profile,
-                                                                       encoding_option)
-        else: # Image Sequence rendering consumers need to be created a bit differently
-            consumer = renderconsumer.get_img_seq_render_consumer(file_path,
-                                                                  profile,
-                                                                  encoding_option)
-        return consumer
-
-    if widgets.args_panel.use_args_check.get_active() == False:
-        # Using options comboboxes
-        encoding_option_index = widgets.encoding_panel.encoding_selector.widget.get_active()
-        quality_option_index = widgets.encoding_panel.quality_selector.widget.get_active()
-        consumer = renderconsumer.get_render_consumer_for_encoding_and_quality( file_path,
-                                                                                profile,
-                                                                                encoding_option_index,
-                                                                                quality_option_index)
-    else:
-        buf = widgets.args_panel.opts_view.get_buffer()
-        consumer, error = renderconsumer.get_render_consumer_for_text_buffer(file_path,
-                                                                             profile,
-                                                                             buf)
-        if error != None:
-            dialogutils.warning_message("FFMPeg Args Error", error, gui.editor_window.window)
-            return None
-        
-    return consumer
 
 def get_args_vals_list_for_current_selections():
     profile = get_current_profile()
@@ -226,7 +112,12 @@ def get_args_vals_list_for_current_selections():
                                                                                             quality_option_index)
         args_vals_list.append(("ar", str(widgets.encoding_panel.sample_rate_selector.get_selected_rate())))
     else: # Manual args encodings
-        buf = widgets.args_panel.opts_view.get_buffer()
+        if widgets.args_panel.text_buffer == None:
+            # Normal height args panel
+            buf = widgets.args_panel.opts_view.get_buffer()
+        else:
+            # Small heights with dialog for setting args
+            buf = widgets.args_panel.text_buffer
         args_vals_list, error = renderconsumer.get_ffmpeg_opts_args_vals_tuples_list(buf)
     
         if error != None:
@@ -242,8 +133,16 @@ def get_file_path():
     if  widgets.args_panel.use_args_check.get_active() == False:
         extension = widgets.file_panel.extension_label.get_text()
     else:
-        extension = "." +  widgets.args_panel.ext_entry.get_text()
-
+        if widgets.args_panel.text_buffer == None:
+            extension = "." +  widgets.args_panel.ext_entry.get_text()
+        else:
+            # Small height with dialog args setting
+            ext_str = widgets.args_panel.args_edit_window.ext_entry.get_text()
+            if ext_str == "":
+                # dialog is closed
+                print "yyyyyyyyy"
+                ext_str = widgets.args_panel.ext
+            extension = "." + ext_str
     return folder + "/" + filename + extension
 
 
@@ -256,9 +155,13 @@ def create_widgets():
     widgets.render_type_panel = rendergui.RenderTypePanel(_render_type_changed, _preset_selection_changed)
     widgets.profile_panel = rendergui.RenderProfilePanel(_out_profile_changed)
     widgets.encoding_panel = rendergui.RenderEncodingPanel(widgets.file_panel.extension_label)
-    widgets.args_panel = rendergui.RenderArgsPanel(_save_opts_pressed, _load_opts_pressed,
-                                                   _display_selection_in_opts_view)
-
+    if (editorstate.SCREEN_HEIGHT > 898):
+        widgets.args_panel = rendergui.RenderArgsPanel(_save_opts_pressed, _load_opts_pressed,
+                                                       _display_selection_in_opts_view)
+    else:
+        widgets.args_panel = rendergui.RenderArgsPanelSmall(_save_opts_pressed, _load_opts_pressed,
+                                                            _display_selection_in_opts_view)
+        
     # Range, Render, Reset, Render Queue
     widgets.render_button = guiutils.get_render_button()
     widgets.range_cb = rendergui.get_range_selection_combo()
