@@ -87,6 +87,9 @@ black_track_clip = None
 # Track that all audio is mixed down to combine for output.
 AUDIO_MIX_DOWN_TRACK = 0
 
+# Vectorscop and RGB Parade
+SCOPE_MIX_VALUES = [1.0, 0.8, 0.5, 0.2, 0.0]
+_scope_over_lay_mix = 2
 
 class Sequence:
     """
@@ -139,11 +142,11 @@ class Sequence:
         
         self.vectorscope = mlt.Filter(self.profile, "frei0r.vectorscope")
         mltrefhold.hold_ref(self.vectorscope) # ?? is this just some anti-crash hack attempt that was not removed
-        self.vectorscope.set("mix", "0.5")
+        self.vectorscope.set("mix", str(SCOPE_MIX_VALUES[_scope_over_lay_mix]))
         self.vectorscope.set("overlay sides", "0.0") 
         self.rgbparade =  mlt.Filter(self.profile, "frei0r.rgbparade")
         mltrefhold.hold_ref(self.rgbparade) # ?? is this just some anti-crash hack attempt that was not removed
-        self.rgbparade.set("mix", "0.4")
+        self.rgbparade.set("mix", str(SCOPE_MIX_VALUES[_scope_over_lay_mix]))
         self.rgbparade.set("overlay sides", "0.0")
         self.outputfilter = None
 
@@ -362,13 +365,23 @@ class Sequence:
                 return False
         return True
 
+    def is_empty(self):
+        for i in range (1, len(self.tracks) - 1):
+            track = self.tracks[i]
+            if len(track.clips) > 0:
+                return False
+        
+        return True
+
     # -------------------------------------------------- clips
-    def create_file_producer_clip(self, path, new_clip_name=None):
+    def create_file_producer_clip(self, path, new_clip_name=None, novalidate=False):
         """
         Creates MLT Producer and adds attributes to it, but does 
         not add it to track/playlist object.
         """
         producer = mlt.Producer(self.profile, str(path)) # this runs 0.5s+ on some clips
+        if novalidate == True:
+            producer.set("mlt_service", "avformat-novalidate")
         mltrefhold.hold_ref(producer)
         producer.path = path
         producer.filters = []
@@ -513,6 +526,15 @@ class Sequence:
         filter_object.create_mlt_filters(self.profile, clip)
         return filter_object
 
+    def set_all_filters_active_state(self, is_active):
+        for i in range(1, len(self.tracks)):
+            track = self.tracks[i]
+            for clip in track.clips:
+                if clip.is_blanck_clip == False:
+                    for f in clip.filters:
+                        f.active = is_active
+                        f.update_mlt_disabled_value()
+        
     # ------------------------------------------------------ compositors
     def create_compositor(self, compositor_type):
         compositor = mlttransitions.create_compositor(compositor_type)
@@ -633,11 +655,15 @@ class Sequence:
         track = self.tracks[-1] # Always last track
         track.clear() # # TRIM INIT CRASH HACK, see clear_hidden_track there may be blank clip here
         track.clips = []
-
+    
         # Display trimmmed clip on hidden track by creating copy of it.
         # File producer
         if path != None:
-            clip = self.create_file_producer_clip(path)
+            clip = editorstate.get_cached_trim_clip(path)
+            if clip == None:
+                clip = self.create_file_producer_clip(path, None, True)
+                editorstate.add_cached_trim_clip(clip)
+    
             if clip_start_pos > 0:
                 edit._insert_blank(track, 0, clip_start_pos)
                 edit._insert_clip(track, clip, 1, 0, clip.get_length() - 1)
@@ -749,6 +775,16 @@ class Sequence:
             self.tractor.attach(self.rgbparade)
             self.outputfilter = self.rgbparade
 
+    def set_scope_overlay_mix(self, mix_value_index):
+        global _scope_over_lay_mix
+        _scope_over_lay_mix = mix_value_index
+
+        self.vectorscope.set("mix", str(SCOPE_MIX_VALUES[_scope_over_lay_mix]))
+        self.rgbparade.set("mix", str(SCOPE_MIX_VALUES[_scope_over_lay_mix]))
+
+    def get_mix_index(self):
+        return _scope_over_lay_mix
+
     # ---------------------------------------------------- watermark
     def add_watermark(self, watermark_file_path):
         watermark = mlt.Filter(self.profile, "watermark")
