@@ -30,11 +30,14 @@ import os
 from operator import itemgetter
 
 import appconsts
+import clipeffectseditor
+import compositormodes
 import dialogs
 import dialogutils
 import glassbuttons
 import gui
 import guicomponents
+import guiutils
 import edit
 import editevent
 import editorpersistance
@@ -49,6 +52,7 @@ import movemodes
 import mlttransitions
 import render
 import renderconsumer
+import respaths
 import syncsplitevent
 import updater
 import utils
@@ -65,7 +69,7 @@ def _get_new_clip_from_clip_monitor():
     """
     if MONITOR_MEDIA_FILE() == None:
         # Info window here
-        return
+        return None
     
     if MONITOR_MEDIA_FILE().type != appconsts.PATTERN_PRODUCER:
         new_clip = current_sequence().create_file_producer_clip(MONITOR_MEDIA_FILE().path)
@@ -365,6 +369,9 @@ def three_point_overwrite_pressed():
     action = edit.three_point_overwrite_action(data)
     action.do_edit()
 
+    if not editorstate.timeline_visible():
+        updater.display_sequence_in_monitor()
+    
     updater.display_tline_cut_frame(track, range_in)
 
 def range_overwrite_pressed():
@@ -413,9 +420,70 @@ def range_overwrite_pressed():
 
     updater.display_tline_cut_frame(track, track.get_clip_index_at(mark_in_frame))
 
-def resync_button_pressed():
-    syncsplitevent.resync_selected()
+def delete_range_button_pressed():
+    # Get data
+    #track = current_sequence().get_first_active_track()
+    #if editevent.track_lock_check_and_user_info(track, range_overwrite_pressed, "range overwrite"):
+    #    return
+    tracks = []
+    for i in range(1, len(current_sequence().tracks) - 1):
+        track = current_sequence().tracks[i]
+        if track.edit_freedom != appconsts.LOCKED:
+            tracks.append(track)
 
+    if len(tracks) == 0:
+        # all tracks are locked!
+        return
+            
+    # tractor is has mark in and mark
+    mark_in_frame = current_sequence().tractor.mark_in
+    mark_out_frame = current_sequence().tractor.mark_out
+    range_length = mark_out_frame - mark_in_frame + 1 # end is incl.
+    if mark_in_frame == -1 or mark_out_frame == -1:
+        primary_txt = _("Timeline Range not set!")
+        secondary_txt = _("You need to set Timeline Range using Mark In and Mark Out buttons\nto perform this edit.")
+        dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
+        return
+
+    movemodes.clear_selected_clips() # edit consumes selection
+
+    updater.save_monitor_frame = False # hack to not get wrong value saved in MediaFile.current_frame
+
+    data = {"tracks":tracks,
+            "mark_in_frame":mark_in_frame,
+            "mark_out_frame":mark_out_frame + 1} # +1 because mark is displayed and end of frame end this 
+                                                 # confirms to user expectation of
+                                                 # of how this should work
+    action = edit.range_delete_action(data)
+    action.do_edit()
+
+    PLAYER().seek_frame(mark_in_frame)
+    
+def resync_button_pressed():
+    if movemodes.selected_track != -1:
+        syncsplitevent.resync_selected()
+    else:
+        if compositormodes.compositor != None:
+            sync_compositor(compositormodes.compositor)
+
+def sync_compositor(compositor):
+    track = current_sequence().tracks[compositor.transition.b_track] # b_track is source track where origin clip is
+    origin_clip = None
+    for clip in track.clips:
+        if clip.id == compositor.origin_clip_id:
+            origin_clip = clip
+    if origin_clip == None:
+        dialogutils.info_message(_("Origin clip not found!"), 
+                             _("Clip used to create this Compositor has been removed\nor moved to different track."), 
+                             gui.editor_window.window)
+        return
+    clip_index = track.clips.index(origin_clip)
+    clip_start = track.clip_start(clip_index)
+    clip_end = clip_start + origin_clip.clip_out - origin_clip.clip_in
+    data = {"compositor":compositor,"clip_in":clip_start,"clip_out":clip_end}
+    action = edit.move_compositor_action(data)
+    action.do_edit()
+    
 def split_audio_button_pressed():
     if movemodes.selected_track == -1:
         return
@@ -425,7 +493,6 @@ def split_audio_button_pressed():
     for i in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
         
         clip = track.clips[i]
-        print i, clip
         if clip.is_blanck_clip == False:
             clips.append(clip)
 
@@ -607,9 +674,12 @@ def _add_transition_dialog_callback(dialog, response_id, selection_widgets, tran
     if _check_transition_handles((from_part - add_thingy),
                                  transition_data["from_handle"], 
                                  to_part - (1 - add_thingy), 
-                                 transition_data["to_handle"]) == False:
+                                 transition_data["to_handle"],
+                                 length) == False:
         return
     
+    editorstate.transition_length = length
+    
     # Get from in and out frames
     from_in = from_clip.clip_out - from_part + add_thingy
     from_out = from_in + length # or transition will include one frame too many
@@ -647,7 +717,7 @@ def _add_transition_dialog_callback(dialog, response_id, selection_widgets, tran
                                         window_text)
 
 def _transition_render_complete(clip_path):
-    print "render complete"
+    print "Render complete"
 
     global transition_render_data
     transition_index, from_clip, to_clip, track, from_in, to_out, transition_type = transition_render_data
@@ -665,25 +735,66 @@ def _transition_render_complete(clip_path):
     action = edit.add_centered_transition_action(data)
     action.do_edit()
 
-def _check_transition_handles(from_req, from_handle, to_req, to_handle):
+def _check_transition_handles(from_req, from_handle, to_req, to_handle, length):
 
-    if from_req > from_handle:
-        info_text = _("There is not enough material available in the FROM clip after the cut") + \
-                    _("\nto create the transition.\n\n") + \
-                    _("<b>Available:</b> ") + str(from_handle) + _(" frame(s)\n") + \
-                    _("<b>Required:</b> ") + str(from_req) + _(" frame(s)")
-        dialogutils.info_message(_("FROM Clip Handle is too short!"),
-                                 info_text,
-                                 gui.editor_window.window)
-        return False
-    if to_req  > to_handle:
-        info_text = _("There is not enough material available in the TO clip before the cut") + \
-                    _("\nto create the transition.\n\n") + \
-                    _("<b>Available:</b> ") + str(to_handle) + _(" frame(s)\n") + \
-                    _("<b>Required:</b> ") + str(to_req) + _(" frame(s)")
-        dialogutils.info_message(_("TO Clip Handle is too short!"),
-                                 info_text,
-                                 gui.editor_window.window)
+    if from_req > from_handle or to_req  > to_handle:
+        SPACE_TAB = "    "
+        info_text = _("To create a rendered transition you need enough media overlap from both clips!\n\n")
+        first_clip_info = None
+        if from_req > from_handle:
+        
+            first_clip_info = \
+                        _("<b>FIRST CLIP MEDIA OVERLAP:</b>  ") + \
+                        SPACE_TAB + _("Available <b>") + str(from_handle) + _("</b> frame(s), " ) + \
+                        SPACE_TAB + _("Required <b>") + str(from_req) + _("</b> frame(s)")
+
+
+        second_clip_info = None
+        if to_req  > to_handle:
+            second_clip_info = \
+                            _("<b>SECOND CLIP MEDIA OVERLAP:</b> ") + \
+                            SPACE_TAB + _("Available <b>") + str(to_handle) + _("</b> frame(s), ") + \
+                            SPACE_TAB + _("Required <b>") + str(to_req) + _("</b> frame(s) ")
+
+        
+        img = Gtk.Image.new_from_file ((respaths.IMAGE_PATH + "transition_wrong.png"))
+        img2 = Gtk.Image.new_from_file ((respaths.IMAGE_PATH + "transition_right.png"))
+        img2.set_margin_bottom(24)
+
+        label1 = Gtk.Label(_("Current situation, not enought media overlap:"))
+        label1.set_margin_bottom(12)
+        label2 = Gtk.Label(_("You need more media overlap:"))
+        label2.set_margin_bottom(12)
+        label2.set_margin_top(24)
+        label3 = Gtk.Label(info_text)
+        label3.set_use_markup(True)
+        if first_clip_info != None:
+            label4 = Gtk.Label(first_clip_info)
+            label4.set_use_markup(True)
+        if second_clip_info != None:
+            label5 = Gtk.Label(second_clip_info)
+            label5.set_use_markup(True)
+        
+        row1 = guiutils.get_centered_box([label1])
+        row2 = guiutils.get_centered_box([img])
+        row3 = guiutils.get_centered_box([label2])
+        row4 = guiutils.get_centered_box([img2])
+        row5 = guiutils.get_centered_box([label3])
+        
+        rows = [row1, row2, row3, row4]
+
+        
+        if first_clip_info != None:
+            row6 = guiutils.get_left_justified_box([label4])
+            rows.append(row6)
+        if second_clip_info != None:
+            row7 = guiutils.get_left_justified_box([label5])
+            rows.append(row7)
+        
+
+        dialogutils.warning_message_with_panels(_("More media overlap needed to create transition!"), 
+                                                "", gui.editor_window.window, True, dialogutils.dialog_destroy, rows)
+                
         return False
 
     return True
@@ -743,6 +854,9 @@ def _add_fade_dialog_callback(dialog, response_id, selection_widgets, transition
                                  gui.editor_window.window)
         return
 
+
+    editorstate.fade_length = length
+
     # Edit clears selection, get track index before selection is cleared
     clip_index = movemodes.selected_range_in
     movemodes.clear_selected_clips()
@@ -794,9 +908,14 @@ def view_mode_menu_lauched(launcher, event):
     guicomponents.get_monitor_view_popupmenu(launcher, event, _view_mode_menu_item_item_activated)
     
 def _view_mode_menu_item_item_activated(widget, msg):
-    editorstate.current_sequence().set_output_mode(msg)
-    gui.editor_window.view_mode_select.set_pixbuf(msg)
-
+    if msg < 3:
+        editorstate.current_sequence().set_output_mode(msg)
+        gui.editor_window.view_mode_select.set_pixbuf(msg)
+    else:
+        mix_value_index = msg - 3 ## this just done in a bit hackish way, 
+        # see guicomponents.get_monitor_view_popupmenu and sequence.SCOPE_MIX_VALUES
+        editorstate.current_sequence().set_scope_overlay_mix(mix_value_index)
+        
 # ------------------------------------------------------- dialogs    
 def no_monitor_clip_info(parent_window):
     primary_txt = _("No Clip loaded into Monitor")
@@ -946,3 +1065,15 @@ def _marker_add_dialog_callback(dialog, response_id, name_entry):
     current_sequence().markers.append((name, current_frame))
     current_sequence().markers = sorted(current_sequence().markers, key=itemgetter(1))
     updater.repaint_tline()
+    
+
+# ---------------------------------------- timeline edits
+def all_filters_off():
+    current_sequence().set_all_filters_active_state(False)
+    clipeffectseditor.update_stack_view()
+
+def all_filters_on():
+    current_sequence().set_all_filters_active_state(True)
+    clipeffectseditor.update_stack_view()
+
+
