@@ -39,12 +39,14 @@ import editorpersistance
 from editorstate import current_sequence
 from editorstate import timeline_visible
 from editorstate import PLAYER
+from editorstate import PROJECT
 from editorstate import EDIT_MODE
 from editorstate import current_proxy_media_paths
 import editorstate
 import gui
 import respaths
 import sequence
+import snapping
 import trimmodes
 import utils
 import updater
@@ -121,6 +123,9 @@ VIDEO_MUTE_ICON = None
 ALL_MUTE_ICON = None
 MARKER_ICON = None
 LEVELS_RENDER_ICON = None
+SNAP_ICON = None
+KEYBOARD_ICON = None
+CLOSE_MATCH_ICON = None
 
 # tc scale
 TC_POINTER_HEAD = None
@@ -200,6 +205,7 @@ SYNC_OFF_COLOR = (0.77, 0.20, 0.3)
 SYNC_GONE_COLOR = (0.4, 0.4, 0.4)
 
 PROXY_STRIP_COLOR = (0.40, 0.60, 0.82)
+PROXY_STRIP_COLOR_SELECTED = (0.52, 0.72, 0.96)
 
 MARK_COLOR = (0.1, 0.1, 0.1)
 
@@ -212,6 +218,8 @@ FRAME_SCALE_SELECTED_COLOR_GRAD_L = get_multiplied_grad(1, 1, FRAME_SCALE_SELECT
 DARK_FRAME_SCALE_SELECTED_COLOR_GRAD = get_multiplied_grad(0, 1, FRAME_SCALE_COLOR_GRAD, 0.7)
 DARK_FRAME_SCALE_SELECTED_COLOR_GRAD_L = get_multiplied_grad(1, 1, FRAME_SCALE_SELECTED_COLOR_GRAD, GRAD_MULTIPLIER * 0.8) 
 
+ICON_SELECTED_OVERLAY_COLOR = (0.8, 0.8, 1.0, 0.3)
+
 FRAME_SCALE_LINES = (0, 0, 0)
 
 BG_COLOR = (0.5, 0.5, 0.55)#(0.6, 0.6, 0.65)
@@ -229,6 +237,8 @@ OVERLAY_TRIM_COLOR = (0.81, 0.82, 0.3)
 POINTER_TRIANGLE_COLOR = (0.6, 0.7, 0.8, 0.7)
 SHADOW_POINTER_COLOR = (0.5, 0.5, 0.5)
 
+MATCH_FRAME_LINES_COLOR = (0.78, 0.31, 0.31)
+
 BLANK_SELECTED = (0.68, 0.68, 0.74)
 
 TRACK_GRAD_STOP1 = (1, 0.68, 0.68, 0.68, 1) #0.93, 0.93, 0.93, 1)
@@ -267,12 +277,25 @@ fake_current_frame = None
 # Used to draw indicators that tell if more frames are available while trimming
 trim_status = appconsts.ON_BETWEEN_FRAME
 
+# Dict for clip thumbnails path -> image
+clip_thumbnails = {}
+
+# Timeline match image
+match_frame = -1
+match_frame_track_index = -1
+image_on_right = True 
+match_frame_image = None
+match_frame_width = 1
+match_frame_height = 1
+
+
 # ------------------------------------------------------------------- module functions
 def load_icons():
     global FULL_LOCK_ICON, FILTER_CLIP_ICON, VIEW_SIDE_ICON,\
     COMPOSITOR_CLIP_ICON, INSERT_ARROW_ICON, AUDIO_MUTE_ICON, MARKER_ICON, \
     VIDEO_MUTE_ICON, ALL_MUTE_ICON, TRACK_BG_ICON, MUTE_AUDIO_ICON, MUTE_VIDEO_ICON, MUTE_ALL_ICON, \
-    TRACK_ALL_ON_V_ICON, TRACK_ALL_ON_A_ICON, MUTE_AUDIO_A_ICON, TC_POINTER_HEAD, EDIT_INDICATOR, LEVELS_RENDER_ICON
+    TRACK_ALL_ON_V_ICON, TRACK_ALL_ON_A_ICON, MUTE_AUDIO_A_ICON, TC_POINTER_HEAD, EDIT_INDICATOR, \
+    LEVELS_RENDER_ICON, SNAP_ICON, KEYBOARD_ICON, CLOSE_MATCH_ICON
 
     FULL_LOCK_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "full_lock.png")
     FILTER_CLIP_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "filter_clip_icon_sharp.png")
@@ -287,6 +310,9 @@ def load_icons():
     MUTE_VIDEO_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "track_video_mute.png")
     MUTE_ALL_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "track_all_mute.png")
     LEVELS_RENDER_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "audio_levels_render.png")
+    SNAP_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "snap_magnet.png")
+    KEYBOARD_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "keyb_trim.png")
+    CLOSE_MATCH_ICON = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "close_match.png")
 
     MARKER_ICON = _load_pixbuf("marker.png")
     TRACK_ALL_ON_V_ICON = _load_pixbuf("track_all_on_V.png")
@@ -311,6 +337,33 @@ def set_dark_bg_color():
     global BG_COLOR
     BG_COLOR = get_multiplied_color((r, g, b), 1.25)
 
+def set_match_frame(tline_match_frame, track_index, display_on_right):
+    global match_frame, match_frame_track_index, image_on_right, match_frame_image
+    match_frame = tline_match_frame
+    match_frame_track_index = track_index
+    image_on_right = display_on_right
+    match_frame_image = None
+
+def match_frame_close_hit(x, y):
+    if match_frame == -1:
+        return False
+    
+    if image_on_right == True:
+        frame_adj = 0
+        img_pos_adj = 0
+    else:
+        frame_adj = 1
+        img_pos_adj = int(match_frame_width)
+    
+    scale_in = (match_frame + frame_adj - pos) * pix_per_frame
+
+    test_x = scale_in - img_pos_adj + 4
+    test_y = 24
+    if (x >= test_x and  x <= test_x + 12) and (y >= test_y and  y <= test_y + 12):
+        return True
+    
+    return False
+
 def _load_pixbuf(icon_file):
     return cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + icon_file)
 
@@ -489,7 +542,9 @@ def draw_overwrite_overlay(cr, data):
 
     arrow_x = start_x + ((end_x - start_x)/2.0)
     _draw_mode_arrow(cr, arrow_x, y, OVERWRITE_MODE_COLOR)
-
+    
+    _draw_snap(cr, y)
+     
 def _draw_move_overlay(cr, data, y):
     # Get data
     press_frame = data["press_frame"]
@@ -645,6 +700,8 @@ def draw_two_roll_overlay(cr, data):
     cr.line_to(selection_frame_x - radius - bit, track_y + track_height)
     cr.stroke()
 
+    _draw_kb_trim_indicator(cr, selection_frame_x, track_y)
+    
 def draw_one_roll_overlay(cr, data):
     track_height = current_sequence().tracks[data["track"]].height
     track_y = _get_track_y(data["track"])
@@ -704,6 +761,8 @@ def draw_one_roll_overlay(cr, data):
         cr.line_to(selection_frame_x - radius - bit, track_y + track_height)
     cr.stroke()
 
+    _draw_kb_trim_indicator(cr, selection_frame_x, track_y)
+    
 def draw_slide_overlay(cr, data):
     track_height = current_sequence().tracks[data["track"]].height
     track_y = _get_track_y(data["track"])
@@ -764,6 +823,8 @@ def draw_slide_overlay(cr, data):
     cr.set_source_surface(VIEW_SIDE_ICON, x, track_y + 4)
     cr.paint()
 
+    _draw_kb_trim_indicator(cr, x, track_y)
+    
 def draw_clip_end_drag_overlay(cr, data):
     if data["editing_clip_end"] == True:
         end = data["frame"]  - pos
@@ -786,6 +847,8 @@ def draw_clip_end_drag_overlay(cr, data):
     cr.rectangle(scale_in, int(y) + 1.5, int(scale_length), track_height - 2.0)
     cr.stroke()
 
+    _draw_snap(cr, y)
+    
 def draw_compositor_move_overlay(cr, data):
     # Get data
     press_frame = data["press_frame"]
@@ -808,7 +871,9 @@ def draw_compositor_move_overlay(cr, data):
     cr.set_line_width(2.0)
     cr.set_source_rgb(*OVERLAY_COLOR)
     cr.stroke()
-    
+
+    _draw_snap(cr, y)
+
 def draw_compositor_trim(cr, data):
     clip_in = data["clip_in"]
     clip_out = data["clip_out"]
@@ -835,6 +900,8 @@ def draw_compositor_trim(cr, data):
         x = scale_in + scale_length - 26
     _draw_two_arrows(cr, x, y + 4, 4)
 
+    _draw_snap(cr, y)
+    
 def _create_compositor_cairo_path(cr, scale_in, scale_length, y, target_y):
     scale_in = int(scale_in) + 0.5
     scale_length = int(scale_length)
@@ -921,6 +988,17 @@ def _draw_view_icon(cr, x, y):
     cr.set_source_surface(VIEW_SIDE_ICON, x, y)
     cr.paint()
 
+def _draw_snap(cr, y):
+    if snapping.snap_active() == True and snapping.show_magnet_icon == True:
+        cr.set_source_surface(SNAP_ICON, int(snapping.get_snap_x()) - 6, int(y) - 14)
+        cr.paint()
+
+def _draw_kb_trim_indicator(cr, x, y):
+    if trimmodes.submode == trimmodes.KEYB_EDIT_ON:
+        cr.set_source_surface(KEYBOARD_ICON, int(x) - 9, int(y) - 16)
+        cr.paint()
+
+
 # ------------------------------- WIDGETS
 class TimeLineCanvas:
     """
@@ -983,6 +1061,10 @@ class TimeLineCanvas:
             button = 1
         elif (state & Gdk.ModifierType.BUTTON3_MASK):
             button = 3
+        
+        track = get_track(y)
+        x = snapping.get_snapped_x(x, track, self.edit_mode_data)
+            
         self.move_listener(x, y, get_frame(x), button, state)
         
     def _release_event(self, event):
@@ -990,7 +1072,12 @@ class TimeLineCanvas:
         Mouse release callback.
         """
         self.drag_on = False
-        self.release_listener(event.x, event.y, get_frame(event.x), \
+        
+        track = get_track(event.y)
+        x = snapping.get_snapped_x(event.x, track, self.edit_mode_data)
+        snapping.mouse_edit_ended()
+        
+        self.release_listener(x, event.y, get_frame(x), \
                               event.button, event.get_state())
 
     def set_pointer_context(self, x, y):
@@ -1048,6 +1135,9 @@ class TimeLineCanvas:
         if EDIT_MODE() != editorstate.SLIDE_TRIM and fake_current_frame != None:
             PLAYER().seek_frame(fake_current_frame)
             fake_current_frame = None
+        
+        # Draw match frame
+        self.draw_match_frame(cr)
             
         # Draw frame pointer
         if EDIT_MODE() != editorstate.SLIDE_TRIM or PLAYER().looping():
@@ -1072,7 +1162,7 @@ class TimeLineCanvas:
         if self.edit_mode_overlay_draw_func != None:
             self.edit_mode_overlay_draw_func(cr,self.edit_mode_data)
         
-        audiowaveformrenderer.render_queued()
+        audiowaveformrenderer.launch_queued_renders()
 
     def draw_track(self, cr, track, y, width):
         """
@@ -1112,6 +1202,8 @@ class TimeLineCanvas:
 
         proxy_paths = current_proxy_media_paths()
 
+        global clip_thumbnails
+                
         # Draw clips in draw range
         for i in range(start, end):
 
@@ -1167,7 +1259,7 @@ class TimeLineCanvas:
                         else:
                             cr.set_source_rgb(*IMAGE_CLIP_SELECTED_COLOR)
                             clip_bg_col = IMAGE_CLIP_SELECTED_COLOR
-                else:
+                else:# Audio clip
                     if not clip.selected:
                         grad = cairo.LinearGradient (0, y, 0, y + track_height)
                         grad.add_color_stop_rgba(*AUDIO_CLIP_COLOR_GRAD)
@@ -1248,6 +1340,44 @@ class TimeLineCanvas:
                     cr.close_path()
                     cr.fill()
 
+            # Draw video clip icon
+            text_x_add = 0
+            if scale_length > TEXT_MIN and editorstate.display_clip_media_thumbnails:
+                if clip.is_blanck_clip == False and track.type == sequence.VIDEO and \
+                    (clip.media_type == sequence.VIDEO or clip.media_type == sequence.IMAGE 
+                        or clip.media_type == sequence.IMAGE_SEQUENCE):
+                        
+                    text_x_add = 115
+                    cr.save()
+                    try: # paint thumbnail
+                        thumb_img = clip_thumbnails[clip.path]
+                        cr.rectangle(scale_in + 4, y + 3.5, scale_length - 8, track_height - 6)
+                        cr.clip()
+                        cr.set_source_surface(thumb_img,scale_in, y - 20)
+                        cr.paint()
+                    except: # thumbnail not found  in dict, get it pait it
+                        try:
+                            media_file = PROJECT().get_media_file_for_path(clip.path)
+                            thumb_img = media_file.icon
+                            cr.rectangle(scale_in + 4, y + 3.5, scale_length - 8, track_height - 6)
+                            cr.clip()
+                            cr.set_source_surface(thumb_img, scale_in, y - 20)
+                            cr.paint()
+                            clip_thumbnails[clip.path] = thumb_img
+                        except:
+                            pass # This fails for rendered fades and transitions
+                    
+                    if clip.selected:
+                        if scale_length - 8 < appconsts.THUMB_WIDTH:
+                            ow = scale_length - 8 
+                        else:
+                            ow = appconsts.THUMB_WIDTH
+                        cr.rectangle(scale_in + 4, y + 3.5, ow, track_height - 6)
+                        cr.set_source_rgba(*ICON_SELECTED_OVERLAY_COLOR)
+                        cr.fill()
+                                                    
+                    cr.restore()
+                
             # Draw sync stripe
             if scale_length > FILL_MIN: 
                 if clip.sync_data != None:
@@ -1300,7 +1430,11 @@ class TimeLineCanvas:
             # Draw proxy indicator
             if scale_length > FILL_MIN:
                 if (not clip.is_blanck_clip) and proxy_paths.get(clip.path) != None:
-                    cr.set_source_rgb(*PROXY_STRIP_COLOR)
+                    if clip.selected:
+                        cr.set_source_rgb(*PROXY_STRIP_COLOR_SELECTED)
+                    else:
+                        cr.set_source_rgb(*PROXY_STRIP_COLOR)
+
                     cr.rectangle(scale_in, y, scale_length, 8)
                     cr.fill()
 
@@ -1364,11 +1498,15 @@ class TimeLineCanvas:
                 
                 # Draw level bar for each frame in draw range
                 for f in range(draw_first, draw_last, step):
-                    x = media_start_pos_pix + f * pix_per_frame
-                    h = bar_height * clip.waveform_data[f]
-                    if h < 1:
-                        h = 1
-                    cr.rectangle(x, y + y_pad + (bar_height - h), draw_pix_per_frame, h)
+                    try:
+                        x = media_start_pos_pix + f * pix_per_frame
+                        h = bar_height * clip.waveform_data[f]
+                        if h < 1:
+                            h = 1
+                        cr.rectangle(x, y + y_pad + (bar_height - h), draw_pix_per_frame, h)
+                    except:
+                        # This is just dirty fix a when 23.98 fps does not work
+                        break
 
                 cr.fill()
 
@@ -1408,7 +1546,7 @@ class TimeLineCanvas:
                                          cairo.FONT_SLANT_NORMAL,
                                          cairo.FONT_WEIGHT_NORMAL)
                     cr.set_font_size(11)
-                    cr.move_to(scale_in + TEXT_X, y + text_y)
+                    cr.move_to(scale_in + TEXT_X + text_x_add, y + text_y)
                     cr.show_text(clip.name.upper())
                 
                 icon_slot = 0
@@ -1544,7 +1682,75 @@ class TimeLineCanvas:
             cr.arc(parent_x + pad, parent_y + pad, small_radius,  0.0 * degrees, 360.0 * degrees)
             cr.fill()
 
+    def draw_match_frame(self, cr):
+        if match_frame == -1:
+            return
+        
+        global match_frame_image
+        if match_frame_image == None:
+            self.create_match_frame_image_surface()
 
+        if image_on_right == True:
+            dir_mult = 1
+            frame_adj = 0
+            img_pos_adj = 0
+        else:
+            dir_mult = -1
+            frame_adj = 1
+            img_pos_adj = int(match_frame_width)
+        
+        scale_in = (match_frame + frame_adj - pos) * pix_per_frame
+                
+        cr.set_source_surface(match_frame_image, scale_in - img_pos_adj, 20)
+        cr.paint_with_alpha(0.7)
+    
+        cr.set_source_surface(CLOSE_MATCH_ICON, scale_in - img_pos_adj + 4, 24)
+        cr.paint()
+        
+        cr.set_source_rgb(*MATCH_FRAME_LINES_COLOR)
+        cr.set_line_width(2.0)
+        cr.rectangle(int(scale_in) - img_pos_adj, 20, int(match_frame_width), int(match_frame_height))
+        cr.stroke()
+
+        cr.move_to(int(scale_in), 0, )
+        cr.line_to(int(scale_in), int(match_frame_height) + 42)
+        cr.stroke()
+
+        start_y = _get_track_y(match_frame_track_index)
+        end_y = _get_track_y(match_frame_track_index - 1)
+        
+        cr.move_to (int(scale_in) + 8 * dir_mult, start_y)
+        cr.line_to (int(scale_in), start_y)
+        cr.line_to (int(scale_in), end_y + 1)
+        cr.line_to (int(scale_in) + 8 * dir_mult, end_y + 1)
+        cr.set_source_rgb(0.2, 0.2, 0.2)
+        cr.set_line_width(4.0)
+        cr.stroke()
+        
+    def create_match_frame_image_surface(self):
+        # Create non-scaled icon
+        matchframe_path = utils.get_hidden_user_dir_path() + appconsts.MATCH_FRAME
+        icon = cairo.ImageSurface.create_from_png(matchframe_path)
+
+        # Create and return scaled icon
+        allocation = canvas_widget.widget.get_allocation()
+        x, y, w, h = allocation.x, allocation.y, allocation.width, allocation.height
+        profile_screen_ratio = float(PROJECT().profile.width()) / float(PROJECT().profile.height())
+        
+        global match_frame_width, match_frame_height
+        match_frame_height = h - 40
+        match_frame_width = match_frame_height * profile_screen_ratio
+    
+        scaled_icon = cairo.ImageSurface(cairo.FORMAT_ARGB32, int(match_frame_width), int(match_frame_height))
+        cr = cairo.Context(scaled_icon)
+        cr.scale(float(match_frame_width) / float(icon.get_width()), float(match_frame_height) / float(icon.get_height()))
+
+        cr.set_source_surface(icon, 0, 0)
+        cr.paint()
+        
+        global match_frame_image
+        match_frame_image = scaled_icon
+        
 class TimeLineColumn:
     """
     GUI component for displaying and editing track parameters.
@@ -1793,7 +1999,7 @@ class TimeLineFrameScale:
         # Get sequence and frames per second value
         seq = current_sequence()
         fps = seq.profile.fps()
-        
+
         # Selected range
         if seq.tractor.mark_in != -1 and seq.tractor.mark_out != -1:
             in_x = (seq.tractor.mark_in - pos) * pix_per_frame
@@ -1803,7 +2009,21 @@ class TimeLineFrameScale:
             cr.set_source(grad)
             cr.rectangle(in_x,0,out_x-in_x,h)
             cr.fill()
-            
+
+        # Draw start indicator triangles
+        if pos == 0:
+            cr.set_source_rgb(*FRAME_SCALE_LINES)
+            start_y = 1
+            tri_h = 8
+            tri_h_half = tri_h / 2
+            tri_w = 8
+            for i in range(0, 3):
+                cr.move_to (0, start_y + i * tri_h)
+                cr.line_to (tri_w, start_y + i * tri_h + tri_h_half)
+                cr.line_to (0, start_y + i * tri_h + tri_h)
+                cr.close_path();
+                cr.fill()
+
         # Set line attr for frames lines
         cr.set_source_rgb(*FRAME_SCALE_LINES)
         cr.set_line_width(1.0)
