@@ -20,6 +20,9 @@
 
 import datetime
 
+import gi
+gi.require_version('Gtk', '3.0')
+
 from gi.repository import GObject, GLib
 from gi.repository import Gtk, Gdk, GdkPixbuf
 
