@@ -22,6 +22,8 @@
 Module handles user edit events for trim, roll and slip trim modes. 
 """
 
+import traceback
+
 import appconsts
 import dialogutils
 import edit
@@ -29,9 +31,11 @@ import editorpersistance
 import editorstate
 from editorstate import current_sequence
 from editorstate import PLAYER
+from editorstate import EDIT_MODE
 import gui
 import tlinewidgets
 import updater
+import utils
 
 # Default value for pre- and post roll in loop playback
 DEFAULT_LOOP_HALF_LENGTH = 25
@@ -59,6 +63,13 @@ set_exit_mode_func = None
 # This function is set when trim modes are entered to be to the "edit init func for" the entered trim mode.
 set_no_edit_mode_func = None
 
+# Sub modes for handling mouse vs. keyboard edits
+NOTHING_ON = 0
+MOUSE_EDIT_ON = 1
+KEYB_EDIT_ON = 2
+
+submode = NOTHING_ON
+
 
 # ------------------------------------ module functions       
 def _get_trim_edit(track, frame):
@@ -225,6 +236,145 @@ def set_no_edit_trim_mode():
         set_no_edit_mode_func()
 
 
+#----------------------------------------------------- keyboard events
+def left_arrow_pressed(ctrl_pressed):
+    global submode
+    if submode == MOUSE_EDIT_ON:
+        return
+        
+    submode = KEYB_EDIT_ON
+    delta = 1
+    if ctrl_pressed:
+        delta = 10
+        
+    if EDIT_MODE() == editorstate.ONE_ROLL_TRIM:
+        _one_roll_trim_left(delta)
+    elif EDIT_MODE() == editorstate.TWO_ROLL_TRIM:
+        _tworoll_trim_left(delta)
+    elif EDIT_MODE() == editorstate.SLIDE_TRIM:
+        _slide_trim_left(delta)
+        
+def right_arrow_pressed(ctrl_pressed):
+    global submode
+    if submode == MOUSE_EDIT_ON:
+        return
+        
+    submode = KEYB_EDIT_ON
+    delta = 1
+    if ctrl_pressed:
+        delta = 10
+        
+    if EDIT_MODE() == editorstate.ONE_ROLL_TRIM:
+        _one_roll_trim_right(delta)
+    elif EDIT_MODE() == editorstate.TWO_ROLL_TRIM:
+        _tworoll_trim_right(delta)
+    elif EDIT_MODE() == editorstate.SLIDE_TRIM:
+        _slide_trim_right(delta)
+
+def enter_pressed():
+    global submode
+    if submode != KEYB_EDIT_ON:
+        return
+
+    if EDIT_MODE() == editorstate.ONE_ROLL_TRIM:
+        _one_roll_enter_edit()
+    elif EDIT_MODE() == editorstate.TWO_ROLL_TRIM:
+        _tworoll_enter_edit()
+    elif EDIT_MODE() == editorstate.SLIDE_TRIM:
+        _slide_enter_edit()
+
+    submode = NOTHING_ON
+
+def _one_roll_trim_left(delta):
+    # Get legal edit frame for overlay display
+    global edit_data
+    frame = edit_data["selected_frame"] - delta
+    frame = _legalize_one_roll_trim(frame, edit_data["trim_limits"])
+    edit_data["selected_frame"] = frame
+    gui.monitor_widget.set_edit_tline_frame(frame, frame - edit_data["edit_frame"])
+    
+    PLAYER().seek_frame(frame)
+    
+def _one_roll_trim_right(delta):
+    # Get legal edit frame for overlay display
+    global edit_data
+    frame = edit_data["selected_frame"] + delta
+    frame = _legalize_one_roll_trim(frame, edit_data["trim_limits"])
+    edit_data["selected_frame"] = frame
+    gui.monitor_widget.set_edit_tline_frame(frame, frame - edit_data["edit_frame"])
+    
+    PLAYER().seek_frame(frame)
+    
+def _one_roll_enter_edit():
+    frame = edit_data["selected_frame"]
+    _do_one_roll_trim_edit(frame)
+
+def _tworoll_trim_left(delta):
+    global edit_data
+    frame = edit_data["selected_frame"] - delta
+    frame = _legalize_two_roll_trim(frame, edit_data["trim_limits"])
+    edit_data["selected_frame"] = frame
+
+    gui.monitor_widget.set_edit_tline_frame(frame, frame - edit_data["edit_frame"])
+    gui.monitor_widget.update_roll_match_frame()
+    
+    PLAYER().seek_frame(frame)
+
+def _tworoll_trim_right(delta):
+    global edit_data
+    frame = edit_data["selected_frame"] + delta
+    frame = _legalize_two_roll_trim(frame, edit_data["trim_limits"])
+    edit_data["selected_frame"] = frame
+
+    gui.monitor_widget.set_edit_tline_frame(frame, frame - edit_data["edit_frame"])
+    gui.monitor_widget.update_roll_match_frame()
+    
+    PLAYER().seek_frame(frame)
+
+def _tworoll_enter_edit():
+    _do_two_roll_edit(edit_data["selected_frame"])
+
+def _slide_trim_left(delta):
+    global edit_data
+    try:
+        frame = edit_data["keyboard_selected_frame"]
+        edit_data["keyboard_selected_frame"] = frame - delta
+    except:
+        try:
+            edit_data["keyboard_selected_frame"] = edit_data["press_start"] - delta
+        except:
+            trim_limits = edit_data["trim_limits"]
+            edit_data["press_start"] = trim_limits["clip_start"] + 1 # this can be anything the relevant thing here is mouse delta
+            edit_data["keyboard_selected_frame"] = edit_data["press_start"] - delta
+    
+    total_delta = edit_data["press_start"] - edit_data["keyboard_selected_frame"]
+    gui.monitor_widget.set_slip_edit_tline_frame(edit_data["clip"], total_delta)
+        
+    display_frame = _update_slide_trim_for_mouse_frame(edit_data["keyboard_selected_frame"])
+    PLAYER().seek_frame(display_frame)
+
+def _slide_trim_right(delta):
+    global edit_data
+    try:
+        frame = edit_data["keyboard_selected_frame"]
+        edit_data["keyboard_selected_frame"] = frame + delta
+    except:
+        try:
+            edit_data["keyboard_selected_frame"] = edit_data["press_start"] + delta
+        except:
+            trim_limits = edit_data["trim_limits"]
+            edit_data["press_start"] = trim_limits["clip_start"] + 1 # this can be anything the relevant thing here is mouse delta
+            edit_data["keyboard_selected_frame"] = edit_data["press_start"] + delta
+
+    total_delta = edit_data["press_start"] - edit_data["keyboard_selected_frame"]
+    gui.monitor_widget.set_slip_edit_tline_frame(edit_data["clip"], total_delta)
+
+    display_frame = _update_slide_trim_for_mouse_frame(edit_data["keyboard_selected_frame"])
+    PLAYER().seek_frame(display_frame)
+
+def _slide_enter_edit():
+    _do_slide_edit()
+    
 # ------------------------------------- ONE ROLL TRIM EVENTS
 def set_oneroll_mode(track, current_frame=-1, editing_to_clip=None):
     """
@@ -288,7 +438,17 @@ def set_oneroll_mode(track, current_frame=-1, editing_to_clip=None):
         clip = edit_data["from_clip"]
         clip_start = trim_limits["from_start"]
 
-    # Display trim clip
+    # Init trim view layout
+    if track.type == appconsts.VIDEO and clip.media_type != appconsts.PATTERN_PRODUCER:
+        if edit_data["to_side_being_edited"]:
+            gui.monitor_widget.set_start_trim_view(edit_data["from_clip"], clip_start)
+        else:
+            gui.monitor_widget.set_end_trim_view(edit_data["to_clip"], clip_start)
+        gui.monitor_widget.set_edit_tline_frame(current_frame, current_frame - edit_frame)
+    else:
+        gui.monitor_widget.set_default_view()
+   
+    # Set interactive trimview on hidden track
     if clip.media_type != appconsts.PATTERN_PRODUCER:
         current_sequence().display_trim_clip(clip.path, clip_start) # file producer
     else:
@@ -301,7 +461,7 @@ def oneroll_trim_press(event, frame):
     """
     User presses mouse when in one roll mode.
     """
-    global mouse_disabled
+    global mouse_disabled, submode
 
     if not _pressed_on_edited_track(event.y):
         track = tlinewidgets.get_track(event.y)
@@ -312,6 +472,7 @@ def oneroll_trim_press(event, frame):
             else:
                 set_no_edit_mode_func() # further mouse events are handled at editevent.py
         else:
+            submode = MOUSE_EDIT_ON # to stop entering keyboard edits until mouse released
             if not editorpersistance.prefs.quick_enter_trims:
                 # new trim inited, editing non-active until release
                 tlinewidgets.trim_mode_in_non_active_state = True
@@ -333,6 +494,7 @@ def oneroll_trim_press(event, frame):
             else:
                 set_no_edit_mode_func() # no furter mouse events will come here
         else:
+            submode = MOUSE_EDIT_ON # to stop entering keyboard edits until mouse released
             if not editorpersistance.prefs.quick_enter_trims:
                 # new trim inited, editing non-active until release
                 tlinewidgets.trim_mode_in_non_active_state = True
@@ -363,14 +525,16 @@ def oneroll_trim_move(x, y, frame, state):
     global edit_data
     frame = _legalize_one_roll_trim(frame, edit_data["trim_limits"])
     edit_data["selected_frame"] = frame
-    
+    gui.monitor_widget.set_edit_tline_frame(frame, frame - edit_data["edit_frame"])
+
     PLAYER().seek_frame(frame)
     
 def oneroll_trim_release(x, y, frame, state):
     """
     User releases mouse when in one roll mode.
     """
-    global mouse_disabled
+    global mouse_disabled, submode
+    submode = NOTHING_ON # we can now enter keyboard edits 
     if mouse_disabled:
         mouse_disabled = False
         # we may have been in non active state because the clip being edited was changed
@@ -378,7 +542,9 @@ def oneroll_trim_release(x, y, frame, state):
         tlinewidgets.trim_mode_in_non_active_state = False 
         gui.tline_canvas.widget.queue_draw()
         return
-
+    
+    gui.monitor_widget.one_roll_mouse_release(edit_data["edit_frame"], frame - edit_data["edit_frame"])
+    
     _do_one_roll_trim_edit(frame)
 
 def _do_one_roll_trim_edit(frame):
@@ -500,12 +666,14 @@ def _pressed_on_one_roll_active_area(frame):
 def set_tworoll_mode(track, current_frame = -1):
     """
     Sets two roll mode
-    """
+    """     
     if track == None:
         return False
     
+    current_frame_trim_view_fix = 0
     if current_frame == -1:
         current_frame = PLAYER().producer.frame() + 1 # +1 because cut frame selects previous clip
+        current_frame_trim_view_fix = -1 # when initing trim view the +1 for current frame needs to be undone
 
     if current_frame >= track.get_length():
         return False
@@ -562,6 +730,19 @@ def set_tworoll_mode(track, current_frame = -1):
     else:
         clip = edit_data["from_clip"]
         clip_start = trim_limits["from_start"]
+
+    # Init two roll trim view layout
+    if track.type == appconsts.VIDEO and clip.media_type != appconsts.PATTERN_PRODUCER:
+        if edit_data["to_side_being_edited"]:
+            gui.monitor_widget.set_roll_trim_right_active_view(edit_data["from_clip"], clip_start)
+        else:
+            gui.monitor_widget.set_roll_trim_left_active_view(edit_data["to_clip"], clip_start)
+        gui.monitor_widget.set_edit_tline_frame(current_frame + current_frame_trim_view_fix, 
+                                                current_frame + current_frame_trim_view_fix - edit_frame)
+    else:
+        gui.monitor_widget.set_default_view()
+
+    # Set interactive trim view clip on hidden track
     if clip.media_type != appconsts.PATTERN_PRODUCER:
         current_sequence().display_trim_clip(clip.path, clip_start) # File producer
     else:
@@ -589,7 +770,9 @@ def tworoll_trim_press(event, frame):
         _attempt_reinit_tworoll(event, frame)
         return
 
-    global edit_data
+    global edit_data, submode
+    submode = MOUSE_EDIT_ON
+
     frame = _legalize_two_roll_trim(frame, edit_data["trim_limits"])
     edit_data["selected_frame"] = frame
     PLAYER().seek_frame(frame)
@@ -626,6 +809,9 @@ def tworoll_trim_move(x, y, frame, state):
     frame = _legalize_two_roll_trim(frame, edit_data["trim_limits"])
     edit_data["selected_frame"] = frame
 
+    gui.monitor_widget.set_edit_tline_frame(frame, frame - edit_data["edit_frame"])
+    gui.monitor_widget.update_roll_match_frame()
+
     PLAYER().seek_frame(frame)
     
 def tworoll_trim_release(x, y, frame, state):
@@ -641,9 +827,15 @@ def tworoll_trim_release(x, y, frame, state):
         mouse_disabled = False
         return
 
-    global edit_data
+    global edit_data, submode
+    submode = NOTHING_ON # we can now enter keyboard edits
+     
     frame = _legalize_two_roll_trim(frame, edit_data["trim_limits"])
     edit_data["selected_frame"] = frame
+
+    gui.monitor_widget.set_edit_tline_frame(frame, frame - edit_data["edit_frame"])
+    gui.monitor_widget.update_roll_match_frame()
+    
     _do_two_roll_edit(frame)
 
 def tworoll_play_pressed():
@@ -701,7 +893,7 @@ def two_rolledit_done(was_redo, cut_frame, delta, track, to_side_being_edited):
     # Calculated frame always reinits in to side, so we need to 
     # step one back to reinit on from side if we did the edit from that side
     if to_side_being_edited != True:
-        frame = frame - 1
+        frame = frame - 2
         if frame < 0:
             frame = 0
 
@@ -766,10 +958,10 @@ def _get_two_roll_first_and_last():
                  
     return (first, last)
 
-#---------------------------------------- SLIDE ROLL TRIM EVENTS
-def set_slide_mode(track, current_frame):
+#---------------------------------------- SLIP ROLL TRIM EVENTS
+def set_slide_mode(track, current_frame): # we need to change to to correct one some time
     """
-    Sets two roll mode
+    Sets SLIP tool mode
     """
     if track == None:
         return None
@@ -808,10 +1000,23 @@ def set_slide_mode(track, current_frame):
     clip = edit_data["clip"]
     clip_start = 0 # we'll calculate the offset from actual position of clip on timeline to display the frame displayed after sliding
 
+    # Init two roll trim view layout
+    if track.type == appconsts.VIDEO and clip.media_type != appconsts.PATTERN_PRODUCER:
+        if not start_frame_being_viewed:
+            gui.monitor_widget.set_slip_trim_right_active_view(edit_data["clip"])
+            gui.monitor_widget.set_edit_tline_frame(clip.clip_out, 0)
+        else:
+            gui.monitor_widget.set_slip_trim_left_active_view(edit_data["clip"])
+            gui.monitor_widget.set_edit_tline_frame(clip.clip_in, 0)
+    else:
+        gui.monitor_widget.set_default_view()
+
+    # Set interactive trim view clip on hidden track
     if clip.media_type != appconsts.PATTERN_PRODUCER:
         current_sequence().display_trim_clip(clip.path, clip_start) # File producer
     else:
         current_sequence().display_trim_clip(None, clip_start, clip.create_data) # pattern producer
+        
     if start_frame_being_viewed:
         PLAYER().seek_frame(clip.clip_in)
     else:
@@ -883,6 +1088,9 @@ def slide_trim_press(event, frame):
         _attempt_reinit_slide(event, frame)
         return
 
+    global submode
+    submode = MOUSE_EDIT_ON
+    
     display_frame = _update_slide_trim_for_mouse_frame(frame)
     PLAYER().seek_frame(display_frame)
 
@@ -890,6 +1098,9 @@ def slide_trim_move(x, y, frame, state):
     if mouse_disabled:
         return
 
+    mouse_delta = edit_data["press_start"] - frame
+    gui.monitor_widget.set_slip_edit_tline_frame(edit_data["clip"], mouse_delta)
+        
     display_frame = _update_slide_trim_for_mouse_frame(frame)
     PLAYER().seek_frame(display_frame)
 
@@ -906,7 +1117,8 @@ def slide_trim_release(x, y, frame, state):
     display_frame = _update_slide_trim_for_mouse_frame(frame)
     PLAYER().seek_frame(display_frame)
 
-    global edit_data
+    global edit_data, submode
+    submode = NOTHING_ON # we can now enter keyboard edits
     display_frame = _update_slide_trim_for_mouse_frame(frame)
     PLAYER().seek_frame(display_frame)
     _do_slide_edit()
