@@ -21,7 +21,7 @@
 """
 Helper functions and data
 """
-
+import time
 
 from gi.repository import Gtk
 
@@ -34,6 +34,8 @@ import threading
 import appconsts
 import editorstate
 
+_start_time = 0.0
+
 # ---------------------------------- CLASSES
 class EmptyClass:
     pass
@@ -345,7 +347,7 @@ def do_nothing():
 
 def get_unique_name_for_audio_levels_file(media_file_path, profile):
     size_str = str(os.path.getsize(media_file_path))
-    fps_str = str(profile.fps())
+    fps_str = str(profile.description())
     file_name = md5.new(media_file_path + size_str + fps_str).hexdigest()
     return file_name
     
@@ -357,6 +359,9 @@ def get_hidden_user_dir_path():
     else:
         return os.getenv("HOME") + "/.flowblade/"
 
+def get_phantom_disk_cache_folder():
+    return get_hidden_user_dir_path() +  appconsts.NODE_COMPOSITORS_DIR + "/" + appconsts.PHANTOM_DISK_CACHE_DIR
+
 def get_hidden_screenshot_dir_path():
     return get_hidden_user_dir_path() + "screenshot/"
 
@@ -427,6 +432,23 @@ def is_media_file(file_path):
     else:
         return True
 
+def program_is_installed(program):
+    def is_exe(fpath):
+        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
+
+    fpath, fname = os.path.split(program)
+    if fpath:
+        if is_exe(program):
+            return True
+    else:
+        for path in os.environ["PATH"].split(os.pathsep):
+            path = path.strip('"')
+            exe_file = os.path.join(path, program)
+            if is_exe(exe_file):
+                return True
+
+    return False
+    
 # File exntension lists
 _audio_file_extensions = [  "act",
                             "aif",
@@ -535,3 +557,20 @@ _video_file_extensions = [  "avi",
                             "xvid",
                             "y4m",
                             "yuv"]
+                            
+                            
+def start_timing(msg="start timing"):
+    global _start_time
+    _start_time = time.time()
+    print msg
+
+def elapsed_time(msg="elapsed: ", show_in_millis=True):
+    elapsed_time = time.time() - _start_time
+    if show_in_millis:
+        elapsed_time = round(elapsed_time * 1000.0, 1)
+        unit = "ms"
+    else:
+        unit = "s"
+    
+    print msg + " " + str(elapsed_time) + " " + unit
+
